# coding=utf-8
# *** WARNING: this file was generated by the Pulumi Terraform Bridge (tfgen) Tool. ***
# *** Do not edit by hand unless you're certain you know what you are doing! ***

import json
import warnings
import pulumi
import pulumi.runtime
from typing import Union
from .. import utilities, tables

class Cluster(pulumi.CustomResource):
    cluster_config: pulumi.Output[dict]
    """
    Allows you to configure various aspects of the cluster.
    Structure defined below.

      * `autoscalingConfig` (`dict`) - The autoscaling policy config associated with the cluster.
        Structure defined below.
        * `policyUri` (`str`) - The autoscaling policy used by the cluster.

      * `bucket` (`str`)
      * `encryptionConfig` (`dict`) - The Customer managed encryption keys settings for the cluster.
        Structure defined below.
        * `kms_key_name` (`str`) - The Cloud KMS key name to use for PD disk encryption for
          all instances in the cluster.

      * `endpointConfig` (`dict`) - The config settings for port access on the cluster.
        Structure defined below.
        - - -
        * `enableHttpPortAccess` (`bool`) - The flag to enable http access to specific ports
          on the cluster from external sources (aka Component Gateway). Defaults to false.
        * `httpPorts` (`dict`)

      * `gceClusterConfig` (`dict`) - Common config settings for resources of Google Compute Engine cluster
        instances, applicable to all instances in the cluster. Structure defined below.
        * `internalIpOnly` (`bool`) - By default, clusters are not restricted to internal IP addresses, 
          and will have ephemeral external IP addresses assigned to each instance. If set to true, all
          instances in the cluster will only have internal IP addresses. Note: Private Google Access
          (also known as `privateIpGoogleAccess`) must be enabled on the subnetwork that the cluster
          will be launched in.
        * `metadata` (`dict`) - A map of the Compute Engine metadata entries to add to all instances
          (see [Project and instance metadata](https://cloud.google.com/compute/docs/storing-retrieving-metadata#project_and_instance_metadata)).
        * `network` (`str`) - The name or self_link of the Google Compute Engine
          network to the cluster will be part of. Conflicts with `subnetwork`.
          If neither is specified, this defaults to the "default" network.
        * `service_account` (`str`) - The service account to be used by the Node VMs.
          If not specified, the "default" service account is used.
        * `serviceAccountScopes` (`list`) - The set of Google API scopes
          to be made available on all of the node VMs under the `service_account`
          specified. These can be	either FQDNs, or scope aliases. The following scopes
          must be set if any other scopes are set. They're necessary to ensure the
          correct functioning ofthe cluster, and are set automatically by the API:
        * `subnetwork` (`str`) - The name or self_link of the Google Compute Engine
          subnetwork the cluster will be part of. Conflicts with `network`.
        * `tags` (`list`) - The list of instance tags applied to instances in the cluster.
          Tags are used to identify valid sources or targets for network firewalls.
        * `zone` (`str`) - The GCP zone where your data is stored and used (i.e. where
          the master and the worker nodes will be created in). If `region` is set to 'global' (default)
          then `zone` is mandatory, otherwise GCP is able to make use of [Auto Zone Placement](https://cloud.google.com/dataproc/docs/concepts/auto-zone)
          to determine this automatically for you.
          Note: This setting additionally determines and restricts
          which computing resources are available for use with other configs such as
          `cluster_config.master_config.machine_type` and `cluster_config.worker_config.machine_type`.

      * `initializationActions` (`list`) - Commands to execute on each node after config is completed.
        You can specify multiple versions of these. Structure defined below.
        * `script` (`str`) - The script to be executed during initialization of the cluster.
          The script must be a GCS file with a gs:// prefix.
        * `timeout_sec` (`float`) - The maximum duration (in seconds) which `script` is
          allowed to take to execute its action. GCP will default to a predetermined
          computed value if not set (currently 300).

      * `lifecycleConfig` (`dict`) - The settings for auto deletion cluster schedule.
        Structure defined below.
        * `autoDeleteTime` (`str`) - The time when cluster will be auto-deleted.
          A timestamp in RFC3339 UTC "Zulu" format, accurate to nanoseconds.
          Example: "2014-10-02T15:01:23.045123456Z".
        * `idleDeleteTtl` (`str`) - The duration to keep the cluster alive while idling
          (no jobs running). After this TTL, the cluster will be deleted. Valid range: [10m, 14d].
        * `idleStartTime` (`str`)

      * `masterConfig` (`dict`) - The Google Compute Engine config settings for the master instances
        in a cluster.. Structure defined below.
        * `accelerators` (`list`) - The Compute Engine accelerator configuration for these instances. Can be specified multiple times.
          * `acceleratorCount` (`float`) - The number of the accelerator cards of this type exposed to this instance. Often restricted to one of `1`, `2`, `4`, or `8`.
          * `accelerator_type` (`str`) - The short name of the accelerator type to expose to this instance. For example, `nvidia-tesla-k80`.

        * `diskConfig` (`dict`) - Disk Config
          * `bootDiskSizeGb` (`float`) - Size of the primary disk attached to each preemptible worker node, specified
            in GB. The smallest allowed disk size is 10GB. GCP will default to a predetermined
            computed value if not set (currently 500GB). Note: If SSDs are not
            attached, it also contains the HDFS data blocks and Hadoop working directories.
          * `bootDiskType` (`str`) - The disk type of the primary disk attached to each preemptible worker node.
            One of `"pd-ssd"` or `"pd-standard"`. Defaults to `"pd-standard"`.
          * `numLocalSsds` (`float`) - The amount of local SSD disks that will be
            attached to each preemptible worker node. Defaults to 0.

        * `imageUri` (`str`) - The URI for the image to use for this worker.  See [the guide](https://cloud.google.com/dataproc/docs/guides/dataproc-images)
          for more information.
        * `instanceNames` (`list`)
        * `machine_type` (`str`) - The name of a Google Compute Engine machine type
          to create for the worker nodes. If not specified, GCP will default to a predetermined
          computed value (currently `n1-standard-4`).
        * `min_cpu_platform` (`str`) - The name of a minimum generation of CPU family
          for the master. If not specified, GCP will default to a predetermined computed value
          for each zone. See [the guide](https://cloud.google.com/compute/docs/instances/specify-min-cpu-platform)
          for details about which CPU families are available (and defaulted) for each zone.
        * `numInstances` (`float`) - Specifies the number of preemptible nodes to create.
          Defaults to 0.

      * `preemptibleWorkerConfig` (`dict`) - The Google Compute Engine config settings for the additional (aka
        preemptible) instances in a cluster. Structure defined below.
        * `diskConfig` (`dict`) - Disk Config
          * `bootDiskSizeGb` (`float`) - Size of the primary disk attached to each preemptible worker node, specified
            in GB. The smallest allowed disk size is 10GB. GCP will default to a predetermined
            computed value if not set (currently 500GB). Note: If SSDs are not
            attached, it also contains the HDFS data blocks and Hadoop working directories.
          * `bootDiskType` (`str`) - The disk type of the primary disk attached to each preemptible worker node.
            One of `"pd-ssd"` or `"pd-standard"`. Defaults to `"pd-standard"`.
          * `numLocalSsds` (`float`) - The amount of local SSD disks that will be
            attached to each preemptible worker node. Defaults to 0.

        * `instanceNames` (`list`)
        * `numInstances` (`float`) - Specifies the number of preemptible nodes to create.
          Defaults to 0.

      * `securityConfig` (`dict`) - Security related configuration. Structure defined below.
        * `kerberosConfig` (`dict`) - Kerberos Configuration
          * `crossRealmTrustAdminServer` (`str`) - The admin server (IP or hostname) for the
            remote trusted realm in a cross realm trust relationship.
          * `crossRealmTrustKdc` (`str`) - The KDC (IP or hostname) for the
            remote trusted realm in a cross realm trust relationship.
          * `crossRealmTrustRealm` (`str`) - The remote realm the Dataproc on-cluster KDC will
            trust, should the user enable cross realm trust.
          * `crossRealmTrustSharedPasswordUri` (`str`) - The Cloud Storage URI of a KMS
            encrypted file containing the shared password between the on-cluster Kerberos realm
            and the remote trusted realm, in a cross realm trust relationship.
          * `enableKerberos` (`bool`) - Flag to indicate whether to Kerberize the cluster.
          * `kdcDbKeyUri` (`str`) - The Cloud Storage URI of a KMS encrypted file containing
            the master key of the KDC database.
          * `keyPasswordUri` (`str`) - The Cloud Storage URI of a KMS encrypted file containing
            the password to the user provided key. For the self-signed certificate, this password
            is generated by Dataproc.
          * `keystorePasswordUri` (`str`) - The Cloud Storage URI of a KMS encrypted file containing
            the password to the user provided keystore. For the self-signed certificated, the password
            is generated by Dataproc.
          * `keystoreUri` (`str`) - The Cloud Storage URI of the keystore file used for SSL encryption.
            If not provided, Dataproc will provide a self-signed certificate.
          * `kmsKeyUri` (`str`) - The URI of the KMS key used to encrypt various sensitive files.
          * `realm` (`str`) - The name of the on-cluster Kerberos realm. If not specified, the
            uppercased domain of hostnames will be the realm.
          * `rootPrincipalPasswordUri` (`str`) - The Cloud Storage URI of a KMS encrypted file
            containing the root principal password.
          * `tgtLifetimeHours` (`float`) - The lifetime of the ticket granting ticket, in hours.
          * `truststorePasswordUri` (`str`) - The Cloud Storage URI of a KMS encrypted file
            containing the password to the user provided truststore. For the self-signed
            certificate, this password is generated by Dataproc.
          * `truststoreUri` (`str`) - The Cloud Storage URI of the truststore file used for
            SSL encryption. If not provided, Dataproc will provide a self-signed certificate.

      * `softwareConfig` (`dict`) - The config settings for software inside the cluster.
        Structure defined below.
        * `imageVersion` (`str`) - The Cloud Dataproc image version to use
          for the cluster - this controls the sets of software versions
          installed onto the nodes when you create clusters. If not specified, defaults to the
          latest version. For a list of valid versions see
          [Cloud Dataproc versions](https://cloud.google.com/dataproc/docs/concepts/dataproc-versions)
        * `optionalComponents` (`list`) - The set of optional components to activate on the cluster. 
          Accepted values are:
          * ANACONDA
          * DRUID
          * HBASE
          * HIVE_WEBHCAT
          * JUPYTER
          * KERBEROS
          * PRESTO
          * RANGER
          * SOLR
          * ZEPPELIN
          * ZOOKEEPER
        * `overrideProperties` (`dict`) - A list of override and additional properties (key/value pairs)
          used to modify various aspects of the common configuration files used when creating
          a cluster. For a list of valid properties please see
          [Cluster properties](https://cloud.google.com/dataproc/docs/concepts/cluster-properties)
        * `properties` (`dict`)

      * `stagingBucket` (`str`) - The Cloud Storage staging bucket used to stage files,
        such as Hadoop jars, between client machines and the cluster.
        Note: If you don't explicitly specify a `staging_bucket`
        then GCP will auto create / assign one for you. However, you are not guaranteed
        an auto generated bucket which is solely dedicated to your cluster; it may be shared
        with other clusters in the same region/zone also choosing to use the auto generation
        option.
      * `worker_config` (`dict`) - The Google Compute Engine config settings for the worker instances
        in a cluster.. Structure defined below.
        * `accelerators` (`list`) - The Compute Engine accelerator configuration for these instances. Can be specified multiple times.
          * `acceleratorCount` (`float`) - The number of the accelerator cards of this type exposed to this instance. Often restricted to one of `1`, `2`, `4`, or `8`.
          * `accelerator_type` (`str`) - The short name of the accelerator type to expose to this instance. For example, `nvidia-tesla-k80`.

        * `diskConfig` (`dict`) - Disk Config
          * `bootDiskSizeGb` (`float`) - Size of the primary disk attached to each preemptible worker node, specified
            in GB. The smallest allowed disk size is 10GB. GCP will default to a predetermined
            computed value if not set (currently 500GB). Note: If SSDs are not
            attached, it also contains the HDFS data blocks and Hadoop working directories.
          * `bootDiskType` (`str`) - The disk type of the primary disk attached to each preemptible worker node.
            One of `"pd-ssd"` or `"pd-standard"`. Defaults to `"pd-standard"`.
          * `numLocalSsds` (`float`) - The amount of local SSD disks that will be
            attached to each preemptible worker node. Defaults to 0.

        * `imageUri` (`str`) - The URI for the image to use for this worker.  See [the guide](https://cloud.google.com/dataproc/docs/guides/dataproc-images)
          for more information.
        * `instanceNames` (`list`)
        * `machine_type` (`str`) - The name of a Google Compute Engine machine type
          to create for the worker nodes. If not specified, GCP will default to a predetermined
          computed value (currently `n1-standard-4`).
        * `min_cpu_platform` (`str`) - The name of a minimum generation of CPU family
          for the master. If not specified, GCP will default to a predetermined computed value
          for each zone. See [the guide](https://cloud.google.com/compute/docs/instances/specify-min-cpu-platform)
          for details about which CPU families are available (and defaulted) for each zone.
        * `numInstances` (`float`) - Specifies the number of preemptible nodes to create.
          Defaults to 0.
    """
    labels: pulumi.Output[dict]
    """
    The list of labels (key/value pairs) to be applied to
    instances in the cluster. GCP generates some itself including `goog-dataproc-cluster-name`
    which is the name of the cluster.
    """
    name: pulumi.Output[str]
    """
    The name of the cluster, unique within the project and
    zone.
    """
    project: pulumi.Output[str]
    """
    The ID of the project in which the `cluster` will exist. If it
    is not provided, the provider project is used.
    """
    region: pulumi.Output[str]
    """
    The region in which the cluster and associated nodes will be created in.
    Defaults to `global`.
    """
    def __init__(__self__, resource_name, opts=None, cluster_config=None, labels=None, name=None, project=None, region=None, __props__=None, __name__=None, __opts__=None):
        """
        Manages a Cloud Dataproc cluster resource within GCP. For more information see
        [the official dataproc documentation](https://cloud.google.com/dataproc/).


        !> **Warning:** Due to limitations of the API, all arguments except
        `labels`,`cluster_config.worker_config.num_instances` and `cluster_config.preemptible_worker_config.num_instances` are non-updatable. Changing others will cause recreation of the
        whole cluster!

        ## Example Usage

        ### Basic

        ```python
        import pulumi
        import pulumi_gcp as gcp

        simplecluster = gcp.dataproc.Cluster("simplecluster", region="us-central1")
        ```

        ### Advanced

        ```python
        import pulumi
        import pulumi_gcp as gcp

        mycluster = gcp.dataproc.Cluster("mycluster",
            cluster_config={
                "gceClusterConfig": {
                    "serviceAccountScopes": [
                        "https://www.googleapis.com/auth/monitoring",
                        "useraccounts-ro",
                        "storage-rw",
                        "logging-write",
                    ],
                    "tags": [
                        "foo",
                        "bar",
                    ],
                },
                "initializationAction": [{
                    "script": "gs://dataproc-initialization-actions/stackdriver/stackdriver.sh",
                    "timeout_sec": 500,
                }],
                "masterConfig": {
                    "diskConfig": {
                        "bootDiskSizeGb": 15,
                        "bootDiskType": "pd-ssd",
                    },
                    "machine_type": "n1-standard-1",
                    "numInstances": 1,
                },
                "preemptibleWorkerConfig": {
                    "numInstances": 0,
                },
                "softwareConfig": {
                    "imageVersion": "1.3.7-deb9",
                    "overrideProperties": {
                        "dataproc:dataproc.allow.zero.workers": "true",
                    },
                },
                "stagingBucket": "dataproc-staging-bucket",
                "worker_config": {
                    "diskConfig": {
                        "bootDiskSizeGb": 15,
                        "numLocalSsds": 1,
                    },
                    "machine_type": "n1-standard-1",
                    "min_cpu_platform": "Intel Skylake",
                    "numInstances": 2,
                },
            },
            labels={
                "foo": "bar",
            },
            region="us-central1")
        ```

        ### Using A GPU Accelerator

        ```python
        import pulumi
        import pulumi_gcp as gcp

        accelerated_cluster = gcp.dataproc.Cluster("acceleratedCluster",
            cluster_config={
                "gceClusterConfig": {
                    "zone": "us-central1-a",
                },
                "masterConfig": {
                    "accelerators": [{
                        "acceleratorCount": "1",
                        "accelerator_type": "nvidia-tesla-k80",
                    }],
                },
            },
            region="us-central1")
        ```

        :param str resource_name: The name of the resource.
        :param pulumi.ResourceOptions opts: Options for the resource.
        :param pulumi.Input[dict] cluster_config: Allows you to configure various aspects of the cluster.
               Structure defined below.
        :param pulumi.Input[dict] labels: The list of labels (key/value pairs) to be applied to
               instances in the cluster. GCP generates some itself including `goog-dataproc-cluster-name`
               which is the name of the cluster.
        :param pulumi.Input[str] name: The name of the cluster, unique within the project and
               zone.
        :param pulumi.Input[str] project: The ID of the project in which the `cluster` will exist. If it
               is not provided, the provider project is used.
        :param pulumi.Input[str] region: The region in which the cluster and associated nodes will be created in.
               Defaults to `global`.

        The **cluster_config** object supports the following:

          * `autoscalingConfig` (`pulumi.Input[dict]`) - The autoscaling policy config associated with the cluster.
            Structure defined below.
            * `policyUri` (`pulumi.Input[str]`) - The autoscaling policy used by the cluster.

          * `bucket` (`pulumi.Input[str]`)
          * `encryptionConfig` (`pulumi.Input[dict]`) - The Customer managed encryption keys settings for the cluster.
            Structure defined below.
            * `kms_key_name` (`pulumi.Input[str]`) - The Cloud KMS key name to use for PD disk encryption for
              all instances in the cluster.

          * `endpointConfig` (`pulumi.Input[dict]`) - The config settings for port access on the cluster.
            Structure defined below.
            - - -
            * `enableHttpPortAccess` (`pulumi.Input[bool]`) - The flag to enable http access to specific ports
              on the cluster from external sources (aka Component Gateway). Defaults to false.
            * `httpPorts` (`pulumi.Input[dict]`)

          * `gceClusterConfig` (`pulumi.Input[dict]`) - Common config settings for resources of Google Compute Engine cluster
            instances, applicable to all instances in the cluster. Structure defined below.
            * `internalIpOnly` (`pulumi.Input[bool]`) - By default, clusters are not restricted to internal IP addresses, 
              and will have ephemeral external IP addresses assigned to each instance. If set to true, all
              instances in the cluster will only have internal IP addresses. Note: Private Google Access
              (also known as `privateIpGoogleAccess`) must be enabled on the subnetwork that the cluster
              will be launched in.
            * `metadata` (`pulumi.Input[dict]`) - A map of the Compute Engine metadata entries to add to all instances
              (see [Project and instance metadata](https://cloud.google.com/compute/docs/storing-retrieving-metadata#project_and_instance_metadata)).
            * `network` (`pulumi.Input[str]`) - The name or self_link of the Google Compute Engine
              network to the cluster will be part of. Conflicts with `subnetwork`.
              If neither is specified, this defaults to the "default" network.
            * `service_account` (`pulumi.Input[str]`) - The service account to be used by the Node VMs.
              If not specified, the "default" service account is used.
            * `serviceAccountScopes` (`pulumi.Input[list]`) - The set of Google API scopes
              to be made available on all of the node VMs under the `service_account`
              specified. These can be	either FQDNs, or scope aliases. The following scopes
              must be set if any other scopes are set. They're necessary to ensure the
              correct functioning ofthe cluster, and are set automatically by the API:
            * `subnetwork` (`pulumi.Input[str]`) - The name or self_link of the Google Compute Engine
              subnetwork the cluster will be part of. Conflicts with `network`.
            * `tags` (`pulumi.Input[list]`) - The list of instance tags applied to instances in the cluster.
              Tags are used to identify valid sources or targets for network firewalls.
            * `zone` (`pulumi.Input[str]`) - The GCP zone where your data is stored and used (i.e. where
              the master and the worker nodes will be created in). If `region` is set to 'global' (default)
              then `zone` is mandatory, otherwise GCP is able to make use of [Auto Zone Placement](https://cloud.google.com/dataproc/docs/concepts/auto-zone)
              to determine this automatically for you.
              Note: This setting additionally determines and restricts
              which computing resources are available for use with other configs such as
              `cluster_config.master_config.machine_type` and `cluster_config.worker_config.machine_type`.

          * `initializationActions` (`pulumi.Input[list]`) - Commands to execute on each node after config is completed.
            You can specify multiple versions of these. Structure defined below.
            * `script` (`pulumi.Input[str]`) - The script to be executed during initialization of the cluster.
              The script must be a GCS file with a gs:// prefix.
            * `timeout_sec` (`pulumi.Input[float]`) - The maximum duration (in seconds) which `script` is
              allowed to take to execute its action. GCP will default to a predetermined
              computed value if not set (currently 300).

          * `lifecycleConfig` (`pulumi.Input[dict]`) - The settings for auto deletion cluster schedule.
            Structure defined below.
            * `autoDeleteTime` (`pulumi.Input[str]`) - The time when cluster will be auto-deleted.
              A timestamp in RFC3339 UTC "Zulu" format, accurate to nanoseconds.
              Example: "2014-10-02T15:01:23.045123456Z".
            * `idleDeleteTtl` (`pulumi.Input[str]`) - The duration to keep the cluster alive while idling
              (no jobs running). After this TTL, the cluster will be deleted. Valid range: [10m, 14d].
            * `idleStartTime` (`pulumi.Input[str]`)

          * `masterConfig` (`pulumi.Input[dict]`) - The Google Compute Engine config settings for the master instances
            in a cluster.. Structure defined below.
            * `accelerators` (`pulumi.Input[list]`) - The Compute Engine accelerator configuration for these instances. Can be specified multiple times.
              * `acceleratorCount` (`pulumi.Input[float]`) - The number of the accelerator cards of this type exposed to this instance. Often restricted to one of `1`, `2`, `4`, or `8`.
              * `accelerator_type` (`pulumi.Input[str]`) - The short name of the accelerator type to expose to this instance. For example, `nvidia-tesla-k80`.

            * `diskConfig` (`pulumi.Input[dict]`) - Disk Config
              * `bootDiskSizeGb` (`pulumi.Input[float]`) - Size of the primary disk attached to each preemptible worker node, specified
                in GB. The smallest allowed disk size is 10GB. GCP will default to a predetermined
                computed value if not set (currently 500GB). Note: If SSDs are not
                attached, it also contains the HDFS data blocks and Hadoop working directories.
              * `bootDiskType` (`pulumi.Input[str]`) - The disk type of the primary disk attached to each preemptible worker node.
                One of `"pd-ssd"` or `"pd-standard"`. Defaults to `"pd-standard"`.
              * `numLocalSsds` (`pulumi.Input[float]`) - The amount of local SSD disks that will be
                attached to each preemptible worker node. Defaults to 0.

            * `imageUri` (`pulumi.Input[str]`) - The URI for the image to use for this worker.  See [the guide](https://cloud.google.com/dataproc/docs/guides/dataproc-images)
              for more information.
            * `instanceNames` (`pulumi.Input[list]`)
            * `machine_type` (`pulumi.Input[str]`) - The name of a Google Compute Engine machine type
              to create for the worker nodes. If not specified, GCP will default to a predetermined
              computed value (currently `n1-standard-4`).
            * `min_cpu_platform` (`pulumi.Input[str]`) - The name of a minimum generation of CPU family
              for the master. If not specified, GCP will default to a predetermined computed value
              for each zone. See [the guide](https://cloud.google.com/compute/docs/instances/specify-min-cpu-platform)
              for details about which CPU families are available (and defaulted) for each zone.
            * `numInstances` (`pulumi.Input[float]`) - Specifies the number of preemptible nodes to create.
              Defaults to 0.

          * `preemptibleWorkerConfig` (`pulumi.Input[dict]`) - The Google Compute Engine config settings for the additional (aka
            preemptible) instances in a cluster. Structure defined below.
            * `diskConfig` (`pulumi.Input[dict]`) - Disk Config
              * `bootDiskSizeGb` (`pulumi.Input[float]`) - Size of the primary disk attached to each preemptible worker node, specified
                in GB. The smallest allowed disk size is 10GB. GCP will default to a predetermined
                computed value if not set (currently 500GB). Note: If SSDs are not
                attached, it also contains the HDFS data blocks and Hadoop working directories.
              * `bootDiskType` (`pulumi.Input[str]`) - The disk type of the primary disk attached to each preemptible worker node.
                One of `"pd-ssd"` or `"pd-standard"`. Defaults to `"pd-standard"`.
              * `numLocalSsds` (`pulumi.Input[float]`) - The amount of local SSD disks that will be
                attached to each preemptible worker node. Defaults to 0.

            * `instanceNames` (`pulumi.Input[list]`)
            * `numInstances` (`pulumi.Input[float]`) - Specifies the number of preemptible nodes to create.
              Defaults to 0.

          * `securityConfig` (`pulumi.Input[dict]`) - Security related configuration. Structure defined below.
            * `kerberosConfig` (`pulumi.Input[dict]`) - Kerberos Configuration
              * `crossRealmTrustAdminServer` (`pulumi.Input[str]`) - The admin server (IP or hostname) for the
                remote trusted realm in a cross realm trust relationship.
              * `crossRealmTrustKdc` (`pulumi.Input[str]`) - The KDC (IP or hostname) for the
                remote trusted realm in a cross realm trust relationship.
              * `crossRealmTrustRealm` (`pulumi.Input[str]`) - The remote realm the Dataproc on-cluster KDC will
                trust, should the user enable cross realm trust.
              * `crossRealmTrustSharedPasswordUri` (`pulumi.Input[str]`) - The Cloud Storage URI of a KMS
                encrypted file containing the shared password between the on-cluster Kerberos realm
                and the remote trusted realm, in a cross realm trust relationship.
              * `enableKerberos` (`pulumi.Input[bool]`) - Flag to indicate whether to Kerberize the cluster.
              * `kdcDbKeyUri` (`pulumi.Input[str]`) - The Cloud Storage URI of a KMS encrypted file containing
                the master key of the KDC database.
              * `keyPasswordUri` (`pulumi.Input[str]`) - The Cloud Storage URI of a KMS encrypted file containing
                the password to the user provided key. For the self-signed certificate, this password
                is generated by Dataproc.
              * `keystorePasswordUri` (`pulumi.Input[str]`) - The Cloud Storage URI of a KMS encrypted file containing
                the password to the user provided keystore. For the self-signed certificated, the password
                is generated by Dataproc.
              * `keystoreUri` (`pulumi.Input[str]`) - The Cloud Storage URI of the keystore file used for SSL encryption.
                If not provided, Dataproc will provide a self-signed certificate.
              * `kmsKeyUri` (`pulumi.Input[str]`) - The URI of the KMS key used to encrypt various sensitive files.
              * `realm` (`pulumi.Input[str]`) - The name of the on-cluster Kerberos realm. If not specified, the
                uppercased domain of hostnames will be the realm.
              * `rootPrincipalPasswordUri` (`pulumi.Input[str]`) - The Cloud Storage URI of a KMS encrypted file
                containing the root principal password.
              * `tgtLifetimeHours` (`pulumi.Input[float]`) - The lifetime of the ticket granting ticket, in hours.
              * `truststorePasswordUri` (`pulumi.Input[str]`) - The Cloud Storage URI of a KMS encrypted file
                containing the password to the user provided truststore. For the self-signed
                certificate, this password is generated by Dataproc.
              * `truststoreUri` (`pulumi.Input[str]`) - The Cloud Storage URI of the truststore file used for
                SSL encryption. If not provided, Dataproc will provide a self-signed certificate.

          * `softwareConfig` (`pulumi.Input[dict]`) - The config settings for software inside the cluster.
            Structure defined below.
            * `imageVersion` (`pulumi.Input[str]`) - The Cloud Dataproc image version to use
              for the cluster - this controls the sets of software versions
              installed onto the nodes when you create clusters. If not specified, defaults to the
              latest version. For a list of valid versions see
              [Cloud Dataproc versions](https://cloud.google.com/dataproc/docs/concepts/dataproc-versions)
            * `optionalComponents` (`pulumi.Input[list]`) - The set of optional components to activate on the cluster. 
              Accepted values are:
              * ANACONDA
              * DRUID
              * HBASE
              * HIVE_WEBHCAT
              * JUPYTER
              * KERBEROS
              * PRESTO
              * RANGER
              * SOLR
              * ZEPPELIN
              * ZOOKEEPER
            * `overrideProperties` (`pulumi.Input[dict]`) - A list of override and additional properties (key/value pairs)
              used to modify various aspects of the common configuration files used when creating
              a cluster. For a list of valid properties please see
              [Cluster properties](https://cloud.google.com/dataproc/docs/concepts/cluster-properties)
            * `properties` (`pulumi.Input[dict]`)

          * `stagingBucket` (`pulumi.Input[str]`) - The Cloud Storage staging bucket used to stage files,
            such as Hadoop jars, between client machines and the cluster.
            Note: If you don't explicitly specify a `staging_bucket`
            then GCP will auto create / assign one for you. However, you are not guaranteed
            an auto generated bucket which is solely dedicated to your cluster; it may be shared
            with other clusters in the same region/zone also choosing to use the auto generation
            option.
          * `worker_config` (`pulumi.Input[dict]`) - The Google Compute Engine config settings for the worker instances
            in a cluster.. Structure defined below.
            * `accelerators` (`pulumi.Input[list]`) - The Compute Engine accelerator configuration for these instances. Can be specified multiple times.
              * `acceleratorCount` (`pulumi.Input[float]`) - The number of the accelerator cards of this type exposed to this instance. Often restricted to one of `1`, `2`, `4`, or `8`.
              * `accelerator_type` (`pulumi.Input[str]`) - The short name of the accelerator type to expose to this instance. For example, `nvidia-tesla-k80`.

            * `diskConfig` (`pulumi.Input[dict]`) - Disk Config
              * `bootDiskSizeGb` (`pulumi.Input[float]`) - Size of the primary disk attached to each preemptible worker node, specified
                in GB. The smallest allowed disk size is 10GB. GCP will default to a predetermined
                computed value if not set (currently 500GB). Note: If SSDs are not
                attached, it also contains the HDFS data blocks and Hadoop working directories.
              * `bootDiskType` (`pulumi.Input[str]`) - The disk type of the primary disk attached to each preemptible worker node.
                One of `"pd-ssd"` or `"pd-standard"`. Defaults to `"pd-standard"`.
              * `numLocalSsds` (`pulumi.Input[float]`) - The amount of local SSD disks that will be
                attached to each preemptible worker node. Defaults to 0.

            * `imageUri` (`pulumi.Input[str]`) - The URI for the image to use for this worker.  See [the guide](https://cloud.google.com/dataproc/docs/guides/dataproc-images)
              for more information.
            * `instanceNames` (`pulumi.Input[list]`)
            * `machine_type` (`pulumi.Input[str]`) - The name of a Google Compute Engine machine type
              to create for the worker nodes. If not specified, GCP will default to a predetermined
              computed value (currently `n1-standard-4`).
            * `min_cpu_platform` (`pulumi.Input[str]`) - The name of a minimum generation of CPU family
              for the master. If not specified, GCP will default to a predetermined computed value
              for each zone. See [the guide](https://cloud.google.com/compute/docs/instances/specify-min-cpu-platform)
              for details about which CPU families are available (and defaulted) for each zone.
            * `numInstances` (`pulumi.Input[float]`) - Specifies the number of preemptible nodes to create.
              Defaults to 0.
        """
        if __name__ is not None:
            warnings.warn("explicit use of __name__ is deprecated", DeprecationWarning)
            resource_name = __name__
        if __opts__ is not None:
            warnings.warn("explicit use of __opts__ is deprecated, use 'opts' instead", DeprecationWarning)
            opts = __opts__
        if opts is None:
            opts = pulumi.ResourceOptions()
        if not isinstance(opts, pulumi.ResourceOptions):
            raise TypeError('Expected resource options to be a ResourceOptions instance')
        if opts.version is None:
            opts.version = utilities.get_version()
        if opts.id is None:
            if __props__ is not None:
                raise TypeError('__props__ is only valid when passed in combination with a valid opts.id to get an existing resource')
            __props__ = dict()

            __props__['cluster_config'] = cluster_config
            __props__['labels'] = labels
            __props__['name'] = name
            __props__['project'] = project
            __props__['region'] = region
        super(Cluster, __self__).__init__(
            'gcp:dataproc/cluster:Cluster',
            resource_name,
            __props__,
            opts)

    @staticmethod
    def get(resource_name, id, opts=None, cluster_config=None, labels=None, name=None, project=None, region=None):
        """
        Get an existing Cluster resource's state with the given name, id, and optional extra
        properties used to qualify the lookup.

        :param str resource_name: The unique name of the resulting resource.
        :param str id: The unique provider ID of the resource to lookup.
        :param pulumi.ResourceOptions opts: Options for the resource.
        :param pulumi.Input[dict] cluster_config: Allows you to configure various aspects of the cluster.
               Structure defined below.
        :param pulumi.Input[dict] labels: The list of labels (key/value pairs) to be applied to
               instances in the cluster. GCP generates some itself including `goog-dataproc-cluster-name`
               which is the name of the cluster.
        :param pulumi.Input[str] name: The name of the cluster, unique within the project and
               zone.
        :param pulumi.Input[str] project: The ID of the project in which the `cluster` will exist. If it
               is not provided, the provider project is used.
        :param pulumi.Input[str] region: The region in which the cluster and associated nodes will be created in.
               Defaults to `global`.

        The **cluster_config** object supports the following:

          * `autoscalingConfig` (`pulumi.Input[dict]`) - The autoscaling policy config associated with the cluster.
            Structure defined below.
            * `policyUri` (`pulumi.Input[str]`) - The autoscaling policy used by the cluster.

          * `bucket` (`pulumi.Input[str]`)
          * `encryptionConfig` (`pulumi.Input[dict]`) - The Customer managed encryption keys settings for the cluster.
            Structure defined below.
            * `kms_key_name` (`pulumi.Input[str]`) - The Cloud KMS key name to use for PD disk encryption for
              all instances in the cluster.

          * `endpointConfig` (`pulumi.Input[dict]`) - The config settings for port access on the cluster.
            Structure defined below.
            - - -
            * `enableHttpPortAccess` (`pulumi.Input[bool]`) - The flag to enable http access to specific ports
              on the cluster from external sources (aka Component Gateway). Defaults to false.
            * `httpPorts` (`pulumi.Input[dict]`)

          * `gceClusterConfig` (`pulumi.Input[dict]`) - Common config settings for resources of Google Compute Engine cluster
            instances, applicable to all instances in the cluster. Structure defined below.
            * `internalIpOnly` (`pulumi.Input[bool]`) - By default, clusters are not restricted to internal IP addresses, 
              and will have ephemeral external IP addresses assigned to each instance. If set to true, all
              instances in the cluster will only have internal IP addresses. Note: Private Google Access
              (also known as `privateIpGoogleAccess`) must be enabled on the subnetwork that the cluster
              will be launched in.
            * `metadata` (`pulumi.Input[dict]`) - A map of the Compute Engine metadata entries to add to all instances
              (see [Project and instance metadata](https://cloud.google.com/compute/docs/storing-retrieving-metadata#project_and_instance_metadata)).
            * `network` (`pulumi.Input[str]`) - The name or self_link of the Google Compute Engine
              network to the cluster will be part of. Conflicts with `subnetwork`.
              If neither is specified, this defaults to the "default" network.
            * `service_account` (`pulumi.Input[str]`) - The service account to be used by the Node VMs.
              If not specified, the "default" service account is used.
            * `serviceAccountScopes` (`pulumi.Input[list]`) - The set of Google API scopes
              to be made available on all of the node VMs under the `service_account`
              specified. These can be	either FQDNs, or scope aliases. The following scopes
              must be set if any other scopes are set. They're necessary to ensure the
              correct functioning ofthe cluster, and are set automatically by the API:
            * `subnetwork` (`pulumi.Input[str]`) - The name or self_link of the Google Compute Engine
              subnetwork the cluster will be part of. Conflicts with `network`.
            * `tags` (`pulumi.Input[list]`) - The list of instance tags applied to instances in the cluster.
              Tags are used to identify valid sources or targets for network firewalls.
            * `zone` (`pulumi.Input[str]`) - The GCP zone where your data is stored and used (i.e. where
              the master and the worker nodes will be created in). If `region` is set to 'global' (default)
              then `zone` is mandatory, otherwise GCP is able to make use of [Auto Zone Placement](https://cloud.google.com/dataproc/docs/concepts/auto-zone)
              to determine this automatically for you.
              Note: This setting additionally determines and restricts
              which computing resources are available for use with other configs such as
              `cluster_config.master_config.machine_type` and `cluster_config.worker_config.machine_type`.

          * `initializationActions` (`pulumi.Input[list]`) - Commands to execute on each node after config is completed.
            You can specify multiple versions of these. Structure defined below.
            * `script` (`pulumi.Input[str]`) - The script to be executed during initialization of the cluster.
              The script must be a GCS file with a gs:// prefix.
            * `timeout_sec` (`pulumi.Input[float]`) - The maximum duration (in seconds) which `script` is
              allowed to take to execute its action. GCP will default to a predetermined
              computed value if not set (currently 300).

          * `lifecycleConfig` (`pulumi.Input[dict]`) - The settings for auto deletion cluster schedule.
            Structure defined below.
            * `autoDeleteTime` (`pulumi.Input[str]`) - The time when cluster will be auto-deleted.
              A timestamp in RFC3339 UTC "Zulu" format, accurate to nanoseconds.
              Example: "2014-10-02T15:01:23.045123456Z".
            * `idleDeleteTtl` (`pulumi.Input[str]`) - The duration to keep the cluster alive while idling
              (no jobs running). After this TTL, the cluster will be deleted. Valid range: [10m, 14d].
            * `idleStartTime` (`pulumi.Input[str]`)

          * `masterConfig` (`pulumi.Input[dict]`) - The Google Compute Engine config settings for the master instances
            in a cluster.. Structure defined below.
            * `accelerators` (`pulumi.Input[list]`) - The Compute Engine accelerator configuration for these instances. Can be specified multiple times.
              * `acceleratorCount` (`pulumi.Input[float]`) - The number of the accelerator cards of this type exposed to this instance. Often restricted to one of `1`, `2`, `4`, or `8`.
              * `accelerator_type` (`pulumi.Input[str]`) - The short name of the accelerator type to expose to this instance. For example, `nvidia-tesla-k80`.

            * `diskConfig` (`pulumi.Input[dict]`) - Disk Config
              * `bootDiskSizeGb` (`pulumi.Input[float]`) - Size of the primary disk attached to each preemptible worker node, specified
                in GB. The smallest allowed disk size is 10GB. GCP will default to a predetermined
                computed value if not set (currently 500GB). Note: If SSDs are not
                attached, it also contains the HDFS data blocks and Hadoop working directories.
              * `bootDiskType` (`pulumi.Input[str]`) - The disk type of the primary disk attached to each preemptible worker node.
                One of `"pd-ssd"` or `"pd-standard"`. Defaults to `"pd-standard"`.
              * `numLocalSsds` (`pulumi.Input[float]`) - The amount of local SSD disks that will be
                attached to each preemptible worker node. Defaults to 0.

            * `imageUri` (`pulumi.Input[str]`) - The URI for the image to use for this worker.  See [the guide](https://cloud.google.com/dataproc/docs/guides/dataproc-images)
              for more information.
            * `instanceNames` (`pulumi.Input[list]`)
            * `machine_type` (`pulumi.Input[str]`) - The name of a Google Compute Engine machine type
              to create for the worker nodes. If not specified, GCP will default to a predetermined
              computed value (currently `n1-standard-4`).
            * `min_cpu_platform` (`pulumi.Input[str]`) - The name of a minimum generation of CPU family
              for the master. If not specified, GCP will default to a predetermined computed value
              for each zone. See [the guide](https://cloud.google.com/compute/docs/instances/specify-min-cpu-platform)
              for details about which CPU families are available (and defaulted) for each zone.
            * `numInstances` (`pulumi.Input[float]`) - Specifies the number of preemptible nodes to create.
              Defaults to 0.

          * `preemptibleWorkerConfig` (`pulumi.Input[dict]`) - The Google Compute Engine config settings for the additional (aka
            preemptible) instances in a cluster. Structure defined below.
            * `diskConfig` (`pulumi.Input[dict]`) - Disk Config
              * `bootDiskSizeGb` (`pulumi.Input[float]`) - Size of the primary disk attached to each preemptible worker node, specified
                in GB. The smallest allowed disk size is 10GB. GCP will default to a predetermined
                computed value if not set (currently 500GB). Note: If SSDs are not
                attached, it also contains the HDFS data blocks and Hadoop working directories.
              * `bootDiskType` (`pulumi.Input[str]`) - The disk type of the primary disk attached to each preemptible worker node.
                One of `"pd-ssd"` or `"pd-standard"`. Defaults to `"pd-standard"`.
              * `numLocalSsds` (`pulumi.Input[float]`) - The amount of local SSD disks that will be
                attached to each preemptible worker node. Defaults to 0.

            * `instanceNames` (`pulumi.Input[list]`)
            * `numInstances` (`pulumi.Input[float]`) - Specifies the number of preemptible nodes to create.
              Defaults to 0.

          * `securityConfig` (`pulumi.Input[dict]`) - Security related configuration. Structure defined below.
            * `kerberosConfig` (`pulumi.Input[dict]`) - Kerberos Configuration
              * `crossRealmTrustAdminServer` (`pulumi.Input[str]`) - The admin server (IP or hostname) for the
                remote trusted realm in a cross realm trust relationship.
              * `crossRealmTrustKdc` (`pulumi.Input[str]`) - The KDC (IP or hostname) for the
                remote trusted realm in a cross realm trust relationship.
              * `crossRealmTrustRealm` (`pulumi.Input[str]`) - The remote realm the Dataproc on-cluster KDC will
                trust, should the user enable cross realm trust.
              * `crossRealmTrustSharedPasswordUri` (`pulumi.Input[str]`) - The Cloud Storage URI of a KMS
                encrypted file containing the shared password between the on-cluster Kerberos realm
                and the remote trusted realm, in a cross realm trust relationship.
              * `enableKerberos` (`pulumi.Input[bool]`) - Flag to indicate whether to Kerberize the cluster.
              * `kdcDbKeyUri` (`pulumi.Input[str]`) - The Cloud Storage URI of a KMS encrypted file containing
                the master key of the KDC database.
              * `keyPasswordUri` (`pulumi.Input[str]`) - The Cloud Storage URI of a KMS encrypted file containing
                the password to the user provided key. For the self-signed certificate, this password
                is generated by Dataproc.
              * `keystorePasswordUri` (`pulumi.Input[str]`) - The Cloud Storage URI of a KMS encrypted file containing
                the password to the user provided keystore. For the self-signed certificated, the password
                is generated by Dataproc.
              * `keystoreUri` (`pulumi.Input[str]`) - The Cloud Storage URI of the keystore file used for SSL encryption.
                If not provided, Dataproc will provide a self-signed certificate.
              * `kmsKeyUri` (`pulumi.Input[str]`) - The URI of the KMS key used to encrypt various sensitive files.
              * `realm` (`pulumi.Input[str]`) - The name of the on-cluster Kerberos realm. If not specified, the
                uppercased domain of hostnames will be the realm.
              * `rootPrincipalPasswordUri` (`pulumi.Input[str]`) - The Cloud Storage URI of a KMS encrypted file
                containing the root principal password.
              * `tgtLifetimeHours` (`pulumi.Input[float]`) - The lifetime of the ticket granting ticket, in hours.
              * `truststorePasswordUri` (`pulumi.Input[str]`) - The Cloud Storage URI of a KMS encrypted file
                containing the password to the user provided truststore. For the self-signed
                certificate, this password is generated by Dataproc.
              * `truststoreUri` (`pulumi.Input[str]`) - The Cloud Storage URI of the truststore file used for
                SSL encryption. If not provided, Dataproc will provide a self-signed certificate.

          * `softwareConfig` (`pulumi.Input[dict]`) - The config settings for software inside the cluster.
            Structure defined below.
            * `imageVersion` (`pulumi.Input[str]`) - The Cloud Dataproc image version to use
              for the cluster - this controls the sets of software versions
              installed onto the nodes when you create clusters. If not specified, defaults to the
              latest version. For a list of valid versions see
              [Cloud Dataproc versions](https://cloud.google.com/dataproc/docs/concepts/dataproc-versions)
            * `optionalComponents` (`pulumi.Input[list]`) - The set of optional components to activate on the cluster. 
              Accepted values are:
              * ANACONDA
              * DRUID
              * HBASE
              * HIVE_WEBHCAT
              * JUPYTER
              * KERBEROS
              * PRESTO
              * RANGER
              * SOLR
              * ZEPPELIN
              * ZOOKEEPER
            * `overrideProperties` (`pulumi.Input[dict]`) - A list of override and additional properties (key/value pairs)
              used to modify various aspects of the common configuration files used when creating
              a cluster. For a list of valid properties please see
              [Cluster properties](https://cloud.google.com/dataproc/docs/concepts/cluster-properties)
            * `properties` (`pulumi.Input[dict]`)

          * `stagingBucket` (`pulumi.Input[str]`) - The Cloud Storage staging bucket used to stage files,
            such as Hadoop jars, between client machines and the cluster.
            Note: If you don't explicitly specify a `staging_bucket`
            then GCP will auto create / assign one for you. However, you are not guaranteed
            an auto generated bucket which is solely dedicated to your cluster; it may be shared
            with other clusters in the same region/zone also choosing to use the auto generation
            option.
          * `worker_config` (`pulumi.Input[dict]`) - The Google Compute Engine config settings for the worker instances
            in a cluster.. Structure defined below.
            * `accelerators` (`pulumi.Input[list]`) - The Compute Engine accelerator configuration for these instances. Can be specified multiple times.
              * `acceleratorCount` (`pulumi.Input[float]`) - The number of the accelerator cards of this type exposed to this instance. Often restricted to one of `1`, `2`, `4`, or `8`.
              * `accelerator_type` (`pulumi.Input[str]`) - The short name of the accelerator type to expose to this instance. For example, `nvidia-tesla-k80`.

            * `diskConfig` (`pulumi.Input[dict]`) - Disk Config
              * `bootDiskSizeGb` (`pulumi.Input[float]`) - Size of the primary disk attached to each preemptible worker node, specified
                in GB. The smallest allowed disk size is 10GB. GCP will default to a predetermined
                computed value if not set (currently 500GB). Note: If SSDs are not
                attached, it also contains the HDFS data blocks and Hadoop working directories.
              * `bootDiskType` (`pulumi.Input[str]`) - The disk type of the primary disk attached to each preemptible worker node.
                One of `"pd-ssd"` or `"pd-standard"`. Defaults to `"pd-standard"`.
              * `numLocalSsds` (`pulumi.Input[float]`) - The amount of local SSD disks that will be
                attached to each preemptible worker node. Defaults to 0.

            * `imageUri` (`pulumi.Input[str]`) - The URI for the image to use for this worker.  See [the guide](https://cloud.google.com/dataproc/docs/guides/dataproc-images)
              for more information.
            * `instanceNames` (`pulumi.Input[list]`)
            * `machine_type` (`pulumi.Input[str]`) - The name of a Google Compute Engine machine type
              to create for the worker nodes. If not specified, GCP will default to a predetermined
              computed value (currently `n1-standard-4`).
            * `min_cpu_platform` (`pulumi.Input[str]`) - The name of a minimum generation of CPU family
              for the master. If not specified, GCP will default to a predetermined computed value
              for each zone. See [the guide](https://cloud.google.com/compute/docs/instances/specify-min-cpu-platform)
              for details about which CPU families are available (and defaulted) for each zone.
            * `numInstances` (`pulumi.Input[float]`) - Specifies the number of preemptible nodes to create.
              Defaults to 0.
        """
        opts = pulumi.ResourceOptions.merge(opts, pulumi.ResourceOptions(id=id))

        __props__ = dict()

        __props__["cluster_config"] = cluster_config
        __props__["labels"] = labels
        __props__["name"] = name
        __props__["project"] = project
        __props__["region"] = region
        return Cluster(resource_name, opts=opts, __props__=__props__)
    def translate_output_property(self, prop):
        return tables._CAMEL_TO_SNAKE_CASE_TABLE.get(prop) or prop

    def translate_input_property(self, prop):
        return tables._SNAKE_TO_CAMEL_CASE_TABLE.get(prop) or prop




# coding=utf-8
# *** WARNING: this file was generated by the Pulumi Terraform Bridge (tfgen) Tool. ***
# *** Do not edit by hand unless you're certain you know what you are doing! ***

import json
import warnings
import pulumi
import pulumi.runtime
from typing import Union
from .. import utilities, tables

class BackendServiceSignedUrlKey(pulumi.CustomResource):
    backend_service: pulumi.Output[str]
    """
    The backend service this signed URL key belongs.
    """
    key_value: pulumi.Output[str]
    """
    128-bit key value used for signing the URL. The key value must be a
    valid RFC 4648 Section 5 base64url encoded string.  **Note**: This property is sensitive and will not be displayed in the plan.
    """
    name: pulumi.Output[str]
    """
    Name of the signed URL key.
    """
    project: pulumi.Output[str]
    """
    The ID of the project in which the resource belongs.
    If it is not provided, the provider project is used.
    """
    def __init__(__self__, resource_name, opts=None, backend_service=None, key_value=None, name=None, project=None, __props__=None, __name__=None, __opts__=None):
        """
        A key for signing Cloud CDN signed URLs for Backend Services.


        To get more information about BackendServiceSignedUrlKey, see:

        * [API documentation](https://cloud.google.com/compute/docs/reference/rest/v1/backendServices)
        * How-to Guides
            * [Using Signed URLs](https://cloud.google.com/cdn/docs/using-signed-urls/)

        > **Warning:** All arguments including `key_value` will be stored in the raw
        state as plain-text.

        ## Example Usage

        ### Backend Service Signed Url Key

        ```python
        import pulumi
        import pulumi_gcp as gcp

        webserver = gcp.compute.InstanceTemplate("webserver",
            machine_type="n1-standard-1",
            network_interface=[{
                "network": "default",
            }],
            disk=[{
                "sourceImage": "debian-cloud/debian-9",
                "autoDelete": True,
                "boot": True,
            }])
        webservers = gcp.compute.InstanceGroupManager("webservers",
            version=[{
                "instanceTemplate": webserver.id,
                "name": "primary",
            }],
            base_instance_name="webserver",
            zone="us-central1-f",
            target_size=1)
        default = gcp.compute.HttpHealthCheck("default",
            request_path="/",
            check_interval_sec=1,
            timeout_sec=1)
        example_backend = gcp.compute.BackendService("exampleBackend",
            description="Our company website",
            port_name="http",
            protocol="HTTP",
            timeout_sec=10,
            enable_cdn=True,
            backend=[{
                "group": webservers.instance_group,
            }],
            health_checks=[default.id])
        backend_key = gcp.compute.BackendServiceSignedUrlKey("backendKey",
            key_value="pPsVemX8GM46QVeezid6Rw==",
            backend_service=example_backend.name)
        ```

        :param str resource_name: The name of the resource.
        :param pulumi.ResourceOptions opts: Options for the resource.
        :param pulumi.Input[str] backend_service: The backend service this signed URL key belongs.
        :param pulumi.Input[str] key_value: 128-bit key value used for signing the URL. The key value must be a
               valid RFC 4648 Section 5 base64url encoded string.  **Note**: This property is sensitive and will not be displayed in the plan.
        :param pulumi.Input[str] name: Name of the signed URL key.
        :param pulumi.Input[str] project: The ID of the project in which the resource belongs.
               If it is not provided, the provider project is used.
        """
        if __name__ is not None:
            warnings.warn("explicit use of __name__ is deprecated", DeprecationWarning)
            resource_name = __name__
        if __opts__ is not None:
            warnings.warn("explicit use of __opts__ is deprecated, use 'opts' instead", DeprecationWarning)
            opts = __opts__
        if opts is None:
            opts = pulumi.ResourceOptions()
        if not isinstance(opts, pulumi.ResourceOptions):
            raise TypeError('Expected resource options to be a ResourceOptions instance')
        if opts.version is None:
            opts.version = utilities.get_version()
        if opts.id is None:
            if __props__ is not None:
                raise TypeError('__props__ is only valid when passed in combination with a valid opts.id to get an existing resource')
            __props__ = dict()

            if backend_service is None:
                raise TypeError("Missing required property 'backend_service'")
            __props__['backend_service'] = backend_service
            if key_value is None:
                raise TypeError("Missing required property 'key_value'")
            __props__['key_value'] = key_value
            __props__['name'] = name
            __props__['project'] = project
        super(BackendServiceSignedUrlKey, __self__).__init__(
            'gcp:compute/backendServiceSignedUrlKey:BackendServiceSignedUrlKey',
            resource_name,
            __props__,
            opts)

    @staticmethod
    def get(resource_name, id, opts=None, backend_service=None, key_value=None, name=None, project=None):
        """
        Get an existing BackendServiceSignedUrlKey resource's state with the given name, id, and optional extra
        properties used to qualify the lookup.

        :param str resource_name: The unique name of the resulting resource.
        :param str id: The unique provider ID of the resource to lookup.
        :param pulumi.ResourceOptions opts: Options for the resource.
        :param pulumi.Input[str] backend_service: The backend service this signed URL key belongs.
        :param pulumi.Input[str] key_value: 128-bit key value used for signing the URL. The key value must be a
               valid RFC 4648 Section 5 base64url encoded string.  **Note**: This property is sensitive and will not be displayed in the plan.
        :param pulumi.Input[str] name: Name of the signed URL key.
        :param pulumi.Input[str] project: The ID of the project in which the resource belongs.
               If it is not provided, the provider project is used.
        """
        opts = pulumi.ResourceOptions.merge(opts, pulumi.ResourceOptions(id=id))

        __props__ = dict()

        __props__["backend_service"] = backend_service
        __props__["key_value"] = key_value
        __props__["name"] = name
        __props__["project"] = project
        return BackendServiceSignedUrlKey(resource_name, opts=opts, __props__=__props__)
    def translate_output_property(self, prop):
        return tables._CAMEL_TO_SNAKE_CASE_TABLE.get(prop) or prop

    def translate_input_property(self, prop):
        return tables._SNAKE_TO_CAMEL_CASE_TABLE.get(prop) or prop




# coding=utf-8
# *** WARNING: this file was generated by the Pulumi Terraform Bridge (tfgen) Tool. ***
# *** Do not edit by hand unless you're certain you know what you are doing! ***

import json
import warnings
import pulumi
import pulumi.runtime
from typing import Union
from .. import utilities, tables

class DomainMapping(pulumi.CustomResource):
    location: pulumi.Output[str]
    """
    The location of the cloud run instance. eg us-central1
    """
    metadata: pulumi.Output[dict]
    """
    Metadata associated with this DomainMapping.  Structure is documented below.

      * `annotations` (`dict`) - Annotations is a key value map stored with a resource that
        may be set by external tools to store and retrieve arbitrary metadata. More
        info: http://kubernetes.io/docs/user-guide/annotations
      * `generation` (`float`) - -
        A sequence number representing a specific generation of the desired state.
      * `labels` (`dict`) - Map of string keys and values that can be used to organize and categorize
        (scope and select) objects. May match selectors of replication controllers
        and routes.
        More info: http://kubernetes.io/docs/user-guide/labels
      * `namespace` (`str`) - In Cloud Run the namespace must be equal to either the
        project ID or project number.
      * `resourceVersion` (`str`) - -
        An opaque value that represents the internal version of this object that
        can be used by clients to determine when objects have changed. May be used
        for optimistic concurrency, change detection, and the watch operation on a
        resource or set of resources. They may only be valid for a
        particular resource or set of resources.
        More info:
        https://git.k8s.io/community/contributors/devel/api-conventions.md#concurrency-control-and-consistency
      * `self_link` (`str`) - -
        SelfLink is a URL representing this object.
      * `uid` (`str`) - -
        UID is a unique id generated by the server on successful creation of a resource and is not
        allowed to change on PUT operations.
        More info: http://kubernetes.io/docs/user-guide/identifiers#uids
    """
    name: pulumi.Output[str]
    """
    Name should be a verified domain
    """
    project: pulumi.Output[str]
    """
    The ID of the project in which the resource belongs.
    If it is not provided, the provider project is used.
    """
    spec: pulumi.Output[dict]
    """
    The spec for this DomainMapping.  Structure is documented below.

      * `certificateMode` (`str`) - The mode of the certificate.
      * `forceOverride` (`bool`) - If set, the mapping will override any mapping set before this spec was set.
        It is recommended that the user leaves this empty to receive an error
        warning about a potential conflict and only set it once the respective UI
        has given such a warning.
      * `routeName` (`str`) - The name of the Cloud Run Service that this DomainMapping applies to.
        The route must exist.
    """
    status: pulumi.Output[dict]
    """
    The current status of the DomainMapping.

      * `conditions` (`list`)
        * `message` (`str`)
        * `reason` (`str`)
        * `status` (`str`)
        * `type` (`str`)

      * `mappedRouteName` (`str`)
      * `observedGeneration` (`float`)
      * `resource_records` (`list`)
        * `name` (`str`) - Name should be a verified domain
        * `rrdata` (`str`)
        * `type` (`str`)
    """
    def __init__(__self__, resource_name, opts=None, location=None, metadata=None, name=None, project=None, spec=None, __props__=None, __name__=None, __opts__=None):
        """
        Resource to hold the state and status of a user's domain mapping.


        To get more information about DomainMapping, see:

        * [API documentation](https://cloud.google.com/run/docs/reference/rest/v1alpha1/projects.locations.domainmappings)
        * How-to Guides
            * [Official Documentation](https://cloud.google.com/run/docs/mapping-custom-domains)

        ## Example Usage

        ### Cloud Run Domain Mapping Basic

        ```python
        import pulumi
        import pulumi_gcp as gcp

        default_service = gcp.cloudrun.Service("defaultService",
            location="us-central1",
            metadata={
                "namespace": "my-project-name",
            },
            template={
                "spec": {
                    "containers": [{
                        "image": "gcr.io/cloudrun/hello",
                    }],
                },
            })
        default_domain_mapping = gcp.cloudrun.DomainMapping("defaultDomainMapping",
            location="us-central1",
            metadata={
                "namespace": "my-project-name",
            },
            spec={
                "routeName": default_service.name,
            })
        ```

        :param str resource_name: The name of the resource.
        :param pulumi.ResourceOptions opts: Options for the resource.
        :param pulumi.Input[str] location: The location of the cloud run instance. eg us-central1
        :param pulumi.Input[dict] metadata: Metadata associated with this DomainMapping.  Structure is documented below.
        :param pulumi.Input[str] name: Name should be a verified domain
        :param pulumi.Input[str] project: The ID of the project in which the resource belongs.
               If it is not provided, the provider project is used.
        :param pulumi.Input[dict] spec: The spec for this DomainMapping.  Structure is documented below.

        The **metadata** object supports the following:

          * `annotations` (`pulumi.Input[dict]`) - Annotations is a key value map stored with a resource that
            may be set by external tools to store and retrieve arbitrary metadata. More
            info: http://kubernetes.io/docs/user-guide/annotations
          * `generation` (`pulumi.Input[float]`) - -
            A sequence number representing a specific generation of the desired state.
          * `labels` (`pulumi.Input[dict]`) - Map of string keys and values that can be used to organize and categorize
            (scope and select) objects. May match selectors of replication controllers
            and routes.
            More info: http://kubernetes.io/docs/user-guide/labels
          * `namespace` (`pulumi.Input[str]`) - In Cloud Run the namespace must be equal to either the
            project ID or project number.
          * `resourceVersion` (`pulumi.Input[str]`) - -
            An opaque value that represents the internal version of this object that
            can be used by clients to determine when objects have changed. May be used
            for optimistic concurrency, change detection, and the watch operation on a
            resource or set of resources. They may only be valid for a
            particular resource or set of resources.
            More info:
            https://git.k8s.io/community/contributors/devel/api-conventions.md#concurrency-control-and-consistency
          * `self_link` (`pulumi.Input[str]`) - -
            SelfLink is a URL representing this object.
          * `uid` (`pulumi.Input[str]`) - -
            UID is a unique id generated by the server on successful creation of a resource and is not
            allowed to change on PUT operations.
            More info: http://kubernetes.io/docs/user-guide/identifiers#uids

        The **spec** object supports the following:

          * `certificateMode` (`pulumi.Input[str]`) - The mode of the certificate.
          * `forceOverride` (`pulumi.Input[bool]`) - If set, the mapping will override any mapping set before this spec was set.
            It is recommended that the user leaves this empty to receive an error
            warning about a potential conflict and only set it once the respective UI
            has given such a warning.
          * `routeName` (`pulumi.Input[str]`) - The name of the Cloud Run Service that this DomainMapping applies to.
            The route must exist.
        """
        if __name__ is not None:
            warnings.warn("explicit use of __name__ is deprecated", DeprecationWarning)
            resource_name = __name__
        if __opts__ is not None:
            warnings.warn("explicit use of __opts__ is deprecated, use 'opts' instead", DeprecationWarning)
            opts = __opts__
        if opts is None:
            opts = pulumi.ResourceOptions()
        if not isinstance(opts, pulumi.ResourceOptions):
            raise TypeError('Expected resource options to be a ResourceOptions instance')
        if opts.version is None:
            opts.version = utilities.get_version()
        if opts.id is None:
            if __props__ is not None:
                raise TypeError('__props__ is only valid when passed in combination with a valid opts.id to get an existing resource')
            __props__ = dict()

            if location is None:
                raise TypeError("Missing required property 'location'")
            __props__['location'] = location
            if metadata is None:
                raise TypeError("Missing required property 'metadata'")
            __props__['metadata'] = metadata
            __props__['name'] = name
            __props__['project'] = project
            if spec is None:
                raise TypeError("Missing required property 'spec'")
            __props__['spec'] = spec
            __props__['status'] = None
        super(DomainMapping, __self__).__init__(
            'gcp:cloudrun/domainMapping:DomainMapping',
            resource_name,
            __props__,
            opts)

    @staticmethod
    def get(resource_name, id, opts=None, location=None, metadata=None, name=None, project=None, spec=None, status=None):
        """
        Get an existing DomainMapping resource's state with the given name, id, and optional extra
        properties used to qualify the lookup.

        :param str resource_name: The unique name of the resulting resource.
        :param str id: The unique provider ID of the resource to lookup.
        :param pulumi.ResourceOptions opts: Options for the resource.
        :param pulumi.Input[str] location: The location of the cloud run instance. eg us-central1
        :param pulumi.Input[dict] metadata: Metadata associated with this DomainMapping.  Structure is documented below.
        :param pulumi.Input[str] name: Name should be a verified domain
        :param pulumi.Input[str] project: The ID of the project in which the resource belongs.
               If it is not provided, the provider project is used.
        :param pulumi.Input[dict] spec: The spec for this DomainMapping.  Structure is documented below.
        :param pulumi.Input[dict] status: The current status of the DomainMapping.

        The **metadata** object supports the following:

          * `annotations` (`pulumi.Input[dict]`) - Annotations is a key value map stored with a resource that
            may be set by external tools to store and retrieve arbitrary metadata. More
            info: http://kubernetes.io/docs/user-guide/annotations
          * `generation` (`pulumi.Input[float]`) - -
            A sequence number representing a specific generation of the desired state.
          * `labels` (`pulumi.Input[dict]`) - Map of string keys and values that can be used to organize and categorize
            (scope and select) objects. May match selectors of replication controllers
            and routes.
            More info: http://kubernetes.io/docs/user-guide/labels
          * `namespace` (`pulumi.Input[str]`) - In Cloud Run the namespace must be equal to either the
            project ID or project number.
          * `resourceVersion` (`pulumi.Input[str]`) - -
            An opaque value that represents the internal version of this object that
            can be used by clients to determine when objects have changed. May be used
            for optimistic concurrency, change detection, and the watch operation on a
            resource or set of resources. They may only be valid for a
            particular resource or set of resources.
            More info:
            https://git.k8s.io/community/contributors/devel/api-conventions.md#concurrency-control-and-consistency
          * `self_link` (`pulumi.Input[str]`) - -
            SelfLink is a URL representing this object.
          * `uid` (`pulumi.Input[str]`) - -
            UID is a unique id generated by the server on successful creation of a resource and is not
            allowed to change on PUT operations.
            More info: http://kubernetes.io/docs/user-guide/identifiers#uids

        The **spec** object supports the following:

          * `certificateMode` (`pulumi.Input[str]`) - The mode of the certificate.
          * `forceOverride` (`pulumi.Input[bool]`) - If set, the mapping will override any mapping set before this spec was set.
            It is recommended that the user leaves this empty to receive an error
            warning about a potential conflict and only set it once the respective UI
            has given such a warning.
          * `routeName` (`pulumi.Input[str]`) - The name of the Cloud Run Service that this DomainMapping applies to.
            The route must exist.

        The **status** object supports the following:

          * `conditions` (`pulumi.Input[list]`)
            * `message` (`pulumi.Input[str]`)
            * `reason` (`pulumi.Input[str]`)
            * `status` (`pulumi.Input[str]`)
            * `type` (`pulumi.Input[str]`)

          * `mappedRouteName` (`pulumi.Input[str]`)
          * `observedGeneration` (`pulumi.Input[float]`)
          * `resource_records` (`pulumi.Input[list]`)
            * `name` (`pulumi.Input[str]`) - Name should be a verified domain
            * `rrdata` (`pulumi.Input[str]`)
            * `type` (`pulumi.Input[str]`)
        """
        opts = pulumi.ResourceOptions.merge(opts, pulumi.ResourceOptions(id=id))

        __props__ = dict()

        __props__["location"] = location
        __props__["metadata"] = metadata
        __props__["name"] = name
        __props__["project"] = project
        __props__["spec"] = spec
        __props__["status"] = status
        return DomainMapping(resource_name, opts=opts, __props__=__props__)
    def translate_output_property(self, prop):
        return tables._CAMEL_TO_SNAKE_CASE_TABLE.get(prop) or prop

    def translate_input_property(self, prop):
        return tables._SNAKE_TO_CAMEL_CASE_TABLE.get(prop) or prop




# coding=utf-8
# *** WARNING: this file was generated by the Pulumi Terraform Bridge (tfgen) Tool. ***
# *** Do not edit by hand unless you're certain you know what you are doing! ***

import json
import warnings
import pulumi
import pulumi.runtime
from typing import Union
from .. import utilities, tables

class Router(pulumi.CustomResource):
    bgp: pulumi.Output[dict]
    """
    BGP information specific to this router.  Structure is documented below.

      * `advertise_mode` (`str`) - User-specified flag to indicate which mode to use for advertisement.
      * `advertised_groups` (`list`) - User-specified list of prefix groups to advertise in custom mode.
        This field can only be populated if advertiseMode is CUSTOM and
        is advertised to all peers of the router. These groups will be
        advertised in addition to any specified prefixes. Leave this field
        blank to advertise no custom groups.
        This enum field has the one valid value: ALL_SUBNETS
      * `advertised_ip_ranges` (`list`) - User-specified list of individual IP ranges to advertise in
        custom mode. This field can only be populated if advertiseMode
        is CUSTOM and is advertised to all peers of the router. These IP
        ranges will be advertised in addition to any specified groups.
        Leave this field blank to advertise no custom IP ranges.  Structure is documented below.
        * `description` (`str`) - User-specified description for the IP range.
        * `range` (`str`) - The IP range to advertise. The value must be a
          CIDR-formatted string.

      * `asn` (`float`) - Local BGP Autonomous System Number (ASN). Must be an RFC6996
        private ASN, either 16-bit or 32-bit. The value will be fixed for
        this router resource. All VPN tunnels that link to this router
        will have the same local ASN.
    """
    creation_timestamp: pulumi.Output[str]
    """
    Creation timestamp in RFC3339 text format.
    """
    description: pulumi.Output[str]
    """
    User-specified description for the IP range.
    """
    name: pulumi.Output[str]
    """
    Name of the resource. The name must be 1-63 characters long, and
    comply with RFC1035. Specifically, the name must be 1-63 characters
    long and match the regular expression `a-z?`
    which means the first character must be a lowercase letter, and all
    following characters must be a dash, lowercase letter, or digit,
    except the last character, which cannot be a dash.
    """
    network: pulumi.Output[str]
    """
    A reference to the network to which this router belongs.
    """
    project: pulumi.Output[str]
    """
    The ID of the project in which the resource belongs.
    If it is not provided, the provider project is used.
    """
    region: pulumi.Output[str]
    """
    Region where the router resides.
    """
    self_link: pulumi.Output[str]
    """
    The URI of the created resource.
    """
    def __init__(__self__, resource_name, opts=None, bgp=None, description=None, name=None, network=None, project=None, region=None, __props__=None, __name__=None, __opts__=None):
        """
        Represents a Router resource.


        To get more information about Router, see:

        * [API documentation](https://cloud.google.com/compute/docs/reference/rest/v1/routers)
        * How-to Guides
            * [Google Cloud Router](https://cloud.google.com/router/docs/)

        ## Example Usage

        ### Router Basic

        ```python
        import pulumi
        import pulumi_gcp as gcp

        foobar_network = gcp.compute.Network("foobarNetwork", auto_create_subnetworks=False)
        foobar_router = gcp.compute.Router("foobarRouter",
            network=foobar_network.name,
            bgp={
                "asn": 64514,
                "advertise_mode": "CUSTOM",
                "advertised_groups": ["ALL_SUBNETS"],
                "advertised_ip_ranges": [
                    {
                        "range": "1.2.3.4",
                    },
                    {
                        "range": "6.7.0.0/16",
                    },
                ],
            })
        ```

        :param str resource_name: The name of the resource.
        :param pulumi.ResourceOptions opts: Options for the resource.
        :param pulumi.Input[dict] bgp: BGP information specific to this router.  Structure is documented below.
        :param pulumi.Input[str] description: User-specified description for the IP range.
        :param pulumi.Input[str] name: Name of the resource. The name must be 1-63 characters long, and
               comply with RFC1035. Specifically, the name must be 1-63 characters
               long and match the regular expression `a-z?`
               which means the first character must be a lowercase letter, and all
               following characters must be a dash, lowercase letter, or digit,
               except the last character, which cannot be a dash.
        :param pulumi.Input[str] network: A reference to the network to which this router belongs.
        :param pulumi.Input[str] project: The ID of the project in which the resource belongs.
               If it is not provided, the provider project is used.
        :param pulumi.Input[str] region: Region where the router resides.

        The **bgp** object supports the following:

          * `advertise_mode` (`pulumi.Input[str]`) - User-specified flag to indicate which mode to use for advertisement.
          * `advertised_groups` (`pulumi.Input[list]`) - User-specified list of prefix groups to advertise in custom mode.
            This field can only be populated if advertiseMode is CUSTOM and
            is advertised to all peers of the router. These groups will be
            advertised in addition to any specified prefixes. Leave this field
            blank to advertise no custom groups.
            This enum field has the one valid value: ALL_SUBNETS
          * `advertised_ip_ranges` (`pulumi.Input[list]`) - User-specified list of individual IP ranges to advertise in
            custom mode. This field can only be populated if advertiseMode
            is CUSTOM and is advertised to all peers of the router. These IP
            ranges will be advertised in addition to any specified groups.
            Leave this field blank to advertise no custom IP ranges.  Structure is documented below.
            * `description` (`pulumi.Input[str]`) - User-specified description for the IP range.
            * `range` (`pulumi.Input[str]`) - The IP range to advertise. The value must be a
              CIDR-formatted string.

          * `asn` (`pulumi.Input[float]`) - Local BGP Autonomous System Number (ASN). Must be an RFC6996
            private ASN, either 16-bit or 32-bit. The value will be fixed for
            this router resource. All VPN tunnels that link to this router
            will have the same local ASN.
        """
        if __name__ is not None:
            warnings.warn("explicit use of __name__ is deprecated", DeprecationWarning)
            resource_name = __name__
        if __opts__ is not None:
            warnings.warn("explicit use of __opts__ is deprecated, use 'opts' instead", DeprecationWarning)
            opts = __opts__
        if opts is None:
            opts = pulumi.ResourceOptions()
        if not isinstance(opts, pulumi.ResourceOptions):
            raise TypeError('Expected resource options to be a ResourceOptions instance')
        if opts.version is None:
            opts.version = utilities.get_version()
        if opts.id is None:
            if __props__ is not None:
                raise TypeError('__props__ is only valid when passed in combination with a valid opts.id to get an existing resource')
            __props__ = dict()

            __props__['bgp'] = bgp
            __props__['description'] = description
            __props__['name'] = name
            if network is None:
                raise TypeError("Missing required property 'network'")
            __props__['network'] = network
            __props__['project'] = project
            __props__['region'] = region
            __props__['creation_timestamp'] = None
            __props__['self_link'] = None
        super(Router, __self__).__init__(
            'gcp:compute/router:Router',
            resource_name,
            __props__,
            opts)

    @staticmethod
    def get(resource_name, id, opts=None, bgp=None, creation_timestamp=None, description=None, name=None, network=None, project=None, region=None, self_link=None):
        """
        Get an existing Router resource's state with the given name, id, and optional extra
        properties used to qualify the lookup.

        :param str resource_name: The unique name of the resulting resource.
        :param str id: The unique provider ID of the resource to lookup.
        :param pulumi.ResourceOptions opts: Options for the resource.
        :param pulumi.Input[dict] bgp: BGP information specific to this router.  Structure is documented below.
        :param pulumi.Input[str] creation_timestamp: Creation timestamp in RFC3339 text format.
        :param pulumi.Input[str] description: User-specified description for the IP range.
        :param pulumi.Input[str] name: Name of the resource. The name must be 1-63 characters long, and
               comply with RFC1035. Specifically, the name must be 1-63 characters
               long and match the regular expression `a-z?`
               which means the first character must be a lowercase letter, and all
               following characters must be a dash, lowercase letter, or digit,
               except the last character, which cannot be a dash.
        :param pulumi.Input[str] network: A reference to the network to which this router belongs.
        :param pulumi.Input[str] project: The ID of the project in which the resource belongs.
               If it is not provided, the provider project is used.
        :param pulumi.Input[str] region: Region where the router resides.
        :param pulumi.Input[str] self_link: The URI of the created resource.

        The **bgp** object supports the following:

          * `advertise_mode` (`pulumi.Input[str]`) - User-specified flag to indicate which mode to use for advertisement.
          * `advertised_groups` (`pulumi.Input[list]`) - User-specified list of prefix groups to advertise in custom mode.
            This field can only be populated if advertiseMode is CUSTOM and
            is advertised to all peers of the router. These groups will be
            advertised in addition to any specified prefixes. Leave this field
            blank to advertise no custom groups.
            This enum field has the one valid value: ALL_SUBNETS
          * `advertised_ip_ranges` (`pulumi.Input[list]`) - User-specified list of individual IP ranges to advertise in
            custom mode. This field can only be populated if advertiseMode
            is CUSTOM and is advertised to all peers of the router. These IP
            ranges will be advertised in addition to any specified groups.
            Leave this field blank to advertise no custom IP ranges.  Structure is documented below.
            * `description` (`pulumi.Input[str]`) - User-specified description for the IP range.
            * `range` (`pulumi.Input[str]`) - The IP range to advertise. The value must be a
              CIDR-formatted string.

          * `asn` (`pulumi.Input[float]`) - Local BGP Autonomous System Number (ASN). Must be an RFC6996
            private ASN, either 16-bit or 32-bit. The value will be fixed for
            this router resource. All VPN tunnels that link to this router
            will have the same local ASN.
        """
        opts = pulumi.ResourceOptions.merge(opts, pulumi.ResourceOptions(id=id))

        __props__ = dict()

        __props__["bgp"] = bgp
        __props__["creation_timestamp"] = creation_timestamp
        __props__["description"] = description
        __props__["name"] = name
        __props__["network"] = network
        __props__["project"] = project
        __props__["region"] = region
        __props__["self_link"] = self_link
        return Router(resource_name, opts=opts, __props__=__props__)
    def translate_output_property(self, prop):
        return tables._CAMEL_TO_SNAKE_CASE_TABLE.get(prop) or prop

    def translate_input_property(self, prop):
        return tables._SNAKE_TO_CAMEL_CASE_TABLE.get(prop) or prop




# coding=utf-8
# *** WARNING: this file was generated by the Pulumi Terraform Bridge (tfgen) Tool. ***
# *** Do not edit by hand unless you're certain you know what you are doing! ***

import json
import warnings
import pulumi
import pulumi.runtime
from typing import Union
from .. import utilities, tables

class Reservation(pulumi.CustomResource):
    commitment: pulumi.Output[str]
    """
    Full or partial URL to a parent commitment. This field displays for reservations that are tied to a commitment.
    """
    creation_timestamp: pulumi.Output[str]
    """
    Creation timestamp in RFC3339 text format.
    """
    description: pulumi.Output[str]
    """
    An optional description of this resource.
    """
    name: pulumi.Output[str]
    """
    Name of the resource. Provided by the client when the resource is
    created. The name must be 1-63 characters long, and comply with
    RFC1035. Specifically, the name must be 1-63 characters long and match
    the regular expression `a-z?` which means the
    first character must be a lowercase letter, and all following
    characters must be a dash, lowercase letter, or digit, except the last
    character, which cannot be a dash.
    """
    project: pulumi.Output[str]
    """
    The ID of the project in which the resource belongs.
    If it is not provided, the provider project is used.
    """
    self_link: pulumi.Output[str]
    """
    The URI of the created resource.
    """
    specific_reservation: pulumi.Output[dict]
    """
    Reservation for instances with specific machine shapes.  Structure is documented below.

      * `count` (`float`) - The number of resources that are allocated.
      * `inUseCount` (`float`) - -
        How many instances are in use.
      * `instanceProperties` (`dict`) - The instance properties for the reservation.  Structure is documented below.
        * `guest_accelerators` (`list`) - Guest accelerator type and count.  Structure is documented below.
          * `acceleratorCount` (`float`) - The number of the guest accelerator cards exposed to
            this instance.
          * `accelerator_type` (`str`) - The full or partial URL of the accelerator type to
            attach to this instance. For example:
            `projects/my-project/zones/us-central1-c/acceleratorTypes/nvidia-tesla-p100`
            If you are creating an instance template, specify only the accelerator name.

        * `localSsds` (`list`) - The amount of local ssd to reserve with each instance. This
          reserves disks of type `local-ssd`.  Structure is documented below.
          * `disk_size_gb` (`float`) - The size of the disk in base-2 GB.
          * `interface` (`str`) - The disk interface to use for attaching this disk.

        * `machine_type` (`str`) - The name of the machine type to reserve.
        * `min_cpu_platform` (`str`) - The minimum CPU platform for the reservation. For example,
          `"Intel Skylake"`. See
          the CPU platform availability reference](https://cloud.google.com/compute/docs/instances/specify-min-cpu-platform#availablezones)
          for information on available CPU platforms.
    """
    specific_reservation_required: pulumi.Output[bool]
    """
    When set to true, only VMs that target this reservation by name can
    consume this reservation. Otherwise, it can be consumed by VMs with
    affinity for any reservation. Defaults to false.
    """
    status: pulumi.Output[str]
    """
    The status of the reservation.
    """
    zone: pulumi.Output[str]
    """
    The zone where the reservation is made.
    """
    def __init__(__self__, resource_name, opts=None, description=None, name=None, project=None, specific_reservation=None, specific_reservation_required=None, zone=None, __props__=None, __name__=None, __opts__=None):
        """
        Represents a reservation resource. A reservation ensures that capacity is
        held in a specific zone even if the reserved VMs are not running.

        Reservations apply only to Compute Engine, Cloud Dataproc, and Google
        Kubernetes Engine VM usage.Reservations do not apply to `f1-micro` or
        `g1-small` machine types, preemptible VMs, sole tenant nodes, or other
        services not listed above
        like Cloud SQL and Dataflow.


        To get more information about Reservation, see:

        * [API documentation](https://cloud.google.com/compute/docs/reference/rest/v1/reservations)
        * How-to Guides
            * [Reserving zonal resources](https://cloud.google.com/compute/docs/instances/reserving-zonal-resources)

        ## Example Usage

        ### Reservation Basic

        ```python
        import pulumi
        import pulumi_gcp as gcp

        gce_reservation = gcp.compute.Reservation("gceReservation",
            specific_reservation={
                "count": 1,
                "instanceProperties": {
                    "machine_type": "n2-standard-2",
                    "min_cpu_platform": "Intel Cascade Lake",
                },
            },
            zone="us-central1-a")
        ```

        :param str resource_name: The name of the resource.
        :param pulumi.ResourceOptions opts: Options for the resource.
        :param pulumi.Input[str] description: An optional description of this resource.
        :param pulumi.Input[str] name: Name of the resource. Provided by the client when the resource is
               created. The name must be 1-63 characters long, and comply with
               RFC1035. Specifically, the name must be 1-63 characters long and match
               the regular expression `a-z?` which means the
               first character must be a lowercase letter, and all following
               characters must be a dash, lowercase letter, or digit, except the last
               character, which cannot be a dash.
        :param pulumi.Input[str] project: The ID of the project in which the resource belongs.
               If it is not provided, the provider project is used.
        :param pulumi.Input[dict] specific_reservation: Reservation for instances with specific machine shapes.  Structure is documented below.
        :param pulumi.Input[bool] specific_reservation_required: When set to true, only VMs that target this reservation by name can
               consume this reservation. Otherwise, it can be consumed by VMs with
               affinity for any reservation. Defaults to false.
        :param pulumi.Input[str] zone: The zone where the reservation is made.

        The **specific_reservation** object supports the following:

          * `count` (`pulumi.Input[float]`) - The number of resources that are allocated.
          * `inUseCount` (`pulumi.Input[float]`) - -
            How many instances are in use.
          * `instanceProperties` (`pulumi.Input[dict]`) - The instance properties for the reservation.  Structure is documented below.
            * `guest_accelerators` (`pulumi.Input[list]`) - Guest accelerator type and count.  Structure is documented below.
              * `acceleratorCount` (`pulumi.Input[float]`) - The number of the guest accelerator cards exposed to
                this instance.
              * `accelerator_type` (`pulumi.Input[str]`) - The full or partial URL of the accelerator type to
                attach to this instance. For example:
                `projects/my-project/zones/us-central1-c/acceleratorTypes/nvidia-tesla-p100`
                If you are creating an instance template, specify only the accelerator name.

            * `localSsds` (`pulumi.Input[list]`) - The amount of local ssd to reserve with each instance. This
              reserves disks of type `local-ssd`.  Structure is documented below.
              * `disk_size_gb` (`pulumi.Input[float]`) - The size of the disk in base-2 GB.
              * `interface` (`pulumi.Input[str]`) - The disk interface to use for attaching this disk.

            * `machine_type` (`pulumi.Input[str]`) - The name of the machine type to reserve.
            * `min_cpu_platform` (`pulumi.Input[str]`) - The minimum CPU platform for the reservation. For example,
              `"Intel Skylake"`. See
              the CPU platform availability reference](https://cloud.google.com/compute/docs/instances/specify-min-cpu-platform#availablezones)
              for information on available CPU platforms.
        """
        if __name__ is not None:
            warnings.warn("explicit use of __name__ is deprecated", DeprecationWarning)
            resource_name = __name__
        if __opts__ is not None:
            warnings.warn("explicit use of __opts__ is deprecated, use 'opts' instead", DeprecationWarning)
            opts = __opts__
        if opts is None:
            opts = pulumi.ResourceOptions()
        if not isinstance(opts, pulumi.ResourceOptions):
            raise TypeError('Expected resource options to be a ResourceOptions instance')
        if opts.version is None:
            opts.version = utilities.get_version()
        if opts.id is None:
            if __props__ is not None:
                raise TypeError('__props__ is only valid when passed in combination with a valid opts.id to get an existing resource')
            __props__ = dict()

            __props__['description'] = description
            __props__['name'] = name
            __props__['project'] = project
            if specific_reservation is None:
                raise TypeError("Missing required property 'specific_reservation'")
            __props__['specific_reservation'] = specific_reservation
            __props__['specific_reservation_required'] = specific_reservation_required
            if zone is None:
                raise TypeError("Missing required property 'zone'")
            __props__['zone'] = zone
            __props__['commitment'] = None
            __props__['creation_timestamp'] = None
            __props__['self_link'] = None
            __props__['status'] = None
        super(Reservation, __self__).__init__(
            'gcp:compute/reservation:Reservation',
            resource_name,
            __props__,
            opts)

    @staticmethod
    def get(resource_name, id, opts=None, commitment=None, creation_timestamp=None, description=None, name=None, project=None, self_link=None, specific_reservation=None, specific_reservation_required=None, status=None, zone=None):
        """
        Get an existing Reservation resource's state with the given name, id, and optional extra
        properties used to qualify the lookup.

        :param str resource_name: The unique name of the resulting resource.
        :param str id: The unique provider ID of the resource to lookup.
        :param pulumi.ResourceOptions opts: Options for the resource.
        :param pulumi.Input[str] commitment: Full or partial URL to a parent commitment. This field displays for reservations that are tied to a commitment.
        :param pulumi.Input[str] creation_timestamp: Creation timestamp in RFC3339 text format.
        :param pulumi.Input[str] description: An optional description of this resource.
        :param pulumi.Input[str] name: Name of the resource. Provided by the client when the resource is
               created. The name must be 1-63 characters long, and comply with
               RFC1035. Specifically, the name must be 1-63 characters long and match
               the regular expression `a-z?` which means the
               first character must be a lowercase letter, and all following
               characters must be a dash, lowercase letter, or digit, except the last
               character, which cannot be a dash.
        :param pulumi.Input[str] project: The ID of the project in which the resource belongs.
               If it is not provided, the provider project is used.
        :param pulumi.Input[str] self_link: The URI of the created resource.
        :param pulumi.Input[dict] specific_reservation: Reservation for instances with specific machine shapes.  Structure is documented below.
        :param pulumi.Input[bool] specific_reservation_required: When set to true, only VMs that target this reservation by name can
               consume this reservation. Otherwise, it can be consumed by VMs with
               affinity for any reservation. Defaults to false.
        :param pulumi.Input[str] status: The status of the reservation.
        :param pulumi.Input[str] zone: The zone where the reservation is made.

        The **specific_reservation** object supports the following:

          * `count` (`pulumi.Input[float]`) - The number of resources that are allocated.
          * `inUseCount` (`pulumi.Input[float]`) - -
            How many instances are in use.
          * `instanceProperties` (`pulumi.Input[dict]`) - The instance properties for the reservation.  Structure is documented below.
            * `guest_accelerators` (`pulumi.Input[list]`) - Guest accelerator type and count.  Structure is documented below.
              * `acceleratorCount` (`pulumi.Input[float]`) - The number of the guest accelerator cards exposed to
                this instance.
              * `accelerator_type` (`pulumi.Input[str]`) - The full or partial URL of the accelerator type to
                attach to this instance. For example:
                `projects/my-project/zones/us-central1-c/acceleratorTypes/nvidia-tesla-p100`
                If you are creating an instance template, specify only the accelerator name.

            * `localSsds` (`pulumi.Input[list]`) - The amount of local ssd to reserve with each instance. This
              reserves disks of type `local-ssd`.  Structure is documented below.
              * `disk_size_gb` (`pulumi.Input[float]`) - The size of the disk in base-2 GB.
              * `interface` (`pulumi.Input[str]`) - The disk interface to use for attaching this disk.

            * `machine_type` (`pulumi.Input[str]`) - The name of the machine type to reserve.
            * `min_cpu_platform` (`pulumi.Input[str]`) - The minimum CPU platform for the reservation. For example,
              `"Intel Skylake"`. See
              the CPU platform availability reference](https://cloud.google.com/compute/docs/instances/specify-min-cpu-platform#availablezones)
              for information on available CPU platforms.
        """
        opts = pulumi.ResourceOptions.merge(opts, pulumi.ResourceOptions(id=id))

        __props__ = dict()

        __props__["commitment"] = commitment
        __props__["creation_timestamp"] = creation_timestamp
        __props__["description"] = description
        __props__["name"] = name
        __props__["project"] = project
        __props__["self_link"] = self_link
        __props__["specific_reservation"] = specific_reservation
        __props__["specific_reservation_required"] = specific_reservation_required
        __props__["status"] = status
        __props__["zone"] = zone
        return Reservation(resource_name, opts=opts, __props__=__props__)
    def translate_output_property(self, prop):
        return tables._CAMEL_TO_SNAKE_CASE_TABLE.get(prop) or prop

    def translate_input_property(self, prop):
        return tables._SNAKE_TO_CAMEL_CASE_TABLE.get(prop) or prop




# coding=utf-8
# *** WARNING: this file was generated by the Pulumi Terraform Bridge (tfgen) Tool. ***
# *** Do not edit by hand unless you're certain you know what you are doing! ***

import json
import warnings
import pulumi
import pulumi.runtime
from typing import Union
from .. import utilities, tables

class Policy(pulumi.CustomResource):
    admission_whitelist_patterns: pulumi.Output[list]
    """
    A whitelist of image patterns to exclude from admission rules. If an
    image's name matches a whitelist pattern, the image's admission
    requests will always be permitted regardless of your admission rules.  Structure is documented below.

      * `namePattern` (`str`) - An image name pattern to whitelist, in the form
        `registry/path/to/image`. This supports a trailing * as a
        wildcard, but this is allowed only in text after the registry/
        part.
    """
    cluster_admission_rules: pulumi.Output[list]
    """
    Per-cluster admission rules. An admission rule specifies either that
    all container images used in a pod creation request must be attested
    to by one or more attestors, that all pod creations will be allowed,
    or that all pod creations will be denied. There can be at most one
    admission rule per cluster spec.

      * `cluster` (`str`) - The identifier for this object. Format specified above.
      * `enforcementMode` (`str`) - The action when a pod creation is denied by the admission rule.
      * `evaluationMode` (`str`) - How this admission rule will be evaluated.
      * `requireAttestationsBies` (`list`) - The resource names of the attestors that must attest to a
        container image. If the attestor is in a different project from the
        policy, it should be specified in the format `projects/*/attestors/*`.
        Each attestor must exist before a policy can reference it. To add an
        attestor to a policy the principal issuing the policy change
        request must be able to read the attestor resource.
        Note: this field must be non-empty when the evaluation_mode field
        specifies REQUIRE_ATTESTATION, otherwise it must be empty.
    """
    default_admission_rule: pulumi.Output[dict]
    """
    Default admission rule for a cluster without a per-cluster admission
    rule.  Structure is documented below.

      * `enforcementMode` (`str`) - The action when a pod creation is denied by the admission rule.
      * `evaluationMode` (`str`) - How this admission rule will be evaluated.
      * `requireAttestationsBies` (`list`) - The resource names of the attestors that must attest to a
        container image. If the attestor is in a different project from the
        policy, it should be specified in the format `projects/*/attestors/*`.
        Each attestor must exist before a policy can reference it. To add an
        attestor to a policy the principal issuing the policy change
        request must be able to read the attestor resource.
        Note: this field must be non-empty when the evaluation_mode field
        specifies REQUIRE_ATTESTATION, otherwise it must be empty.
    """
    description: pulumi.Output[str]
    """
    A descriptive comment.
    """
    global_policy_evaluation_mode: pulumi.Output[str]
    """
    Controls the evaluation of a Google-maintained global admission policy
    for common system-level images. Images not covered by the global
    policy will be subject to the project admission policy.
    """
    project: pulumi.Output[str]
    """
    The ID of the project in which the resource belongs.
    If it is not provided, the provider project is used.
    """
    def __init__(__self__, resource_name, opts=None, admission_whitelist_patterns=None, cluster_admission_rules=None, default_admission_rule=None, description=None, global_policy_evaluation_mode=None, project=None, __props__=None, __name__=None, __opts__=None):
        """
        A policy for container image binary authorization.


        To get more information about Policy, see:

        * [API documentation](https://cloud.google.com/binary-authorization/docs/reference/rest/)
        * How-to Guides
            * [Official Documentation](https://cloud.google.com/binary-authorization/)

        ## Example Usage

        ### Binary Authorization Policy Basic

        ```python
        import pulumi
        import pulumi_gcp as gcp

        note = gcp.containeranalysis.Note("note", attestation_authority={
            "hint": {
                "humanReadableName": "My attestor",
            },
        })
        attestor = gcp.binaryauthorization.Attestor("attestor", attestation_authority_note={
            "noteReference": note.name,
        })
        policy = gcp.binaryauthorization.Policy("policy",
            admission_whitelist_patterns=[{
                "namePattern": "gcr.io/google_containers/*",
            }],
            default_admission_rule={
                "evaluationMode": "ALWAYS_ALLOW",
                "enforcementMode": "ENFORCED_BLOCK_AND_AUDIT_LOG",
            },
            cluster_admission_rules=[{
                "cluster": "us-central1-a.prod-cluster",
                "evaluationMode": "REQUIRE_ATTESTATION",
                "enforcementMode": "ENFORCED_BLOCK_AND_AUDIT_LOG",
                "requireAttestationsBies": [attestor.name],
            }])
        ```

        ### Binary Authorization Policy Global Evaluation

        ```python
        import pulumi
        import pulumi_gcp as gcp

        note = gcp.containeranalysis.Note("note", attestation_authority={
            "hint": {
                "humanReadableName": "My attestor",
            },
        })
        attestor = gcp.binaryauthorization.Attestor("attestor", attestation_authority_note={
            "noteReference": note.name,
        })
        policy = gcp.binaryauthorization.Policy("policy",
            default_admission_rule={
                "evaluationMode": "REQUIRE_ATTESTATION",
                "enforcementMode": "ENFORCED_BLOCK_AND_AUDIT_LOG",
                "requireAttestationsBies": [attestor.name],
            },
            global_policy_evaluation_mode="ENABLE")
        ```

        :param str resource_name: The name of the resource.
        :param pulumi.ResourceOptions opts: Options for the resource.
        :param pulumi.Input[list] admission_whitelist_patterns: A whitelist of image patterns to exclude from admission rules. If an
               image's name matches a whitelist pattern, the image's admission
               requests will always be permitted regardless of your admission rules.  Structure is documented below.
        :param pulumi.Input[list] cluster_admission_rules: Per-cluster admission rules. An admission rule specifies either that
               all container images used in a pod creation request must be attested
               to by one or more attestors, that all pod creations will be allowed,
               or that all pod creations will be denied. There can be at most one
               admission rule per cluster spec.
        :param pulumi.Input[dict] default_admission_rule: Default admission rule for a cluster without a per-cluster admission
               rule.  Structure is documented below.
        :param pulumi.Input[str] description: A descriptive comment.
        :param pulumi.Input[str] global_policy_evaluation_mode: Controls the evaluation of a Google-maintained global admission policy
               for common system-level images. Images not covered by the global
               policy will be subject to the project admission policy.
        :param pulumi.Input[str] project: The ID of the project in which the resource belongs.
               If it is not provided, the provider project is used.

        The **admission_whitelist_patterns** object supports the following:

          * `namePattern` (`pulumi.Input[str]`) - An image name pattern to whitelist, in the form
            `registry/path/to/image`. This supports a trailing * as a
            wildcard, but this is allowed only in text after the registry/
            part.

        The **cluster_admission_rules** object supports the following:

          * `cluster` (`pulumi.Input[str]`) - The identifier for this object. Format specified above.
          * `enforcementMode` (`pulumi.Input[str]`) - The action when a pod creation is denied by the admission rule.
          * `evaluationMode` (`pulumi.Input[str]`) - How this admission rule will be evaluated.
          * `requireAttestationsBies` (`pulumi.Input[list]`) - The resource names of the attestors that must attest to a
            container image. If the attestor is in a different project from the
            policy, it should be specified in the format `projects/*/attestors/*`.
            Each attestor must exist before a policy can reference it. To add an
            attestor to a policy the principal issuing the policy change
            request must be able to read the attestor resource.
            Note: this field must be non-empty when the evaluation_mode field
            specifies REQUIRE_ATTESTATION, otherwise it must be empty.

        The **default_admission_rule** object supports the following:

          * `enforcementMode` (`pulumi.Input[str]`) - The action when a pod creation is denied by the admission rule.
          * `evaluationMode` (`pulumi.Input[str]`) - How this admission rule will be evaluated.
          * `requireAttestationsBies` (`pulumi.Input[list]`) - The resource names of the attestors that must attest to a
            container image. If the attestor is in a different project from the
            policy, it should be specified in the format `projects/*/attestors/*`.
            Each attestor must exist before a policy can reference it. To add an
            attestor to a policy the principal issuing the policy change
            request must be able to read the attestor resource.
            Note: this field must be non-empty when the evaluation_mode field
            specifies REQUIRE_ATTESTATION, otherwise it must be empty.
        """
        if __name__ is not None:
            warnings.warn("explicit use of __name__ is deprecated", DeprecationWarning)
            resource_name = __name__
        if __opts__ is not None:
            warnings.warn("explicit use of __opts__ is deprecated, use 'opts' instead", DeprecationWarning)
            opts = __opts__
        if opts is None:
            opts = pulumi.ResourceOptions()
        if not isinstance(opts, pulumi.ResourceOptions):
            raise TypeError('Expected resource options to be a ResourceOptions instance')
        if opts.version is None:
            opts.version = utilities.get_version()
        if opts.id is None:
            if __props__ is not None:
                raise TypeError('__props__ is only valid when passed in combination with a valid opts.id to get an existing resource')
            __props__ = dict()

            __props__['admission_whitelist_patterns'] = admission_whitelist_patterns
            __props__['cluster_admission_rules'] = cluster_admission_rules
            if default_admission_rule is None:
                raise TypeError("Missing required property 'default_admission_rule'")
            __props__['default_admission_rule'] = default_admission_rule
            __props__['description'] = description
            __props__['global_policy_evaluation_mode'] = global_policy_evaluation_mode
            __props__['project'] = project
        super(Policy, __self__).__init__(
            'gcp:binaryauthorization/policy:Policy',
            resource_name,
            __props__,
            opts)

    @staticmethod
    def get(resource_name, id, opts=None, admission_whitelist_patterns=None, cluster_admission_rules=None, default_admission_rule=None, description=None, global_policy_evaluation_mode=None, project=None):
        """
        Get an existing Policy resource's state with the given name, id, and optional extra
        properties used to qualify the lookup.

        :param str resource_name: The unique name of the resulting resource.
        :param str id: The unique provider ID of the resource to lookup.
        :param pulumi.ResourceOptions opts: Options for the resource.
        :param pulumi.Input[list] admission_whitelist_patterns: A whitelist of image patterns to exclude from admission rules. If an
               image's name matches a whitelist pattern, the image's admission
               requests will always be permitted regardless of your admission rules.  Structure is documented below.
        :param pulumi.Input[list] cluster_admission_rules: Per-cluster admission rules. An admission rule specifies either that
               all container images used in a pod creation request must be attested
               to by one or more attestors, that all pod creations will be allowed,
               or that all pod creations will be denied. There can be at most one
               admission rule per cluster spec.
        :param pulumi.Input[dict] default_admission_rule: Default admission rule for a cluster without a per-cluster admission
               rule.  Structure is documented below.
        :param pulumi.Input[str] description: A descriptive comment.
        :param pulumi.Input[str] global_policy_evaluation_mode: Controls the evaluation of a Google-maintained global admission policy
               for common system-level images. Images not covered by the global
               policy will be subject to the project admission policy.
        :param pulumi.Input[str] project: The ID of the project in which the resource belongs.
               If it is not provided, the provider project is used.

        The **admission_whitelist_patterns** object supports the following:

          * `namePattern` (`pulumi.Input[str]`) - An image name pattern to whitelist, in the form
            `registry/path/to/image`. This supports a trailing * as a
            wildcard, but this is allowed only in text after the registry/
            part.

        The **cluster_admission_rules** object supports the following:

          * `cluster` (`pulumi.Input[str]`) - The identifier for this object. Format specified above.
          * `enforcementMode` (`pulumi.Input[str]`) - The action when a pod creation is denied by the admission rule.
          * `evaluationMode` (`pulumi.Input[str]`) - How this admission rule will be evaluated.
          * `requireAttestationsBies` (`pulumi.Input[list]`) - The resource names of the attestors that must attest to a
            container image. If the attestor is in a different project from the
            policy, it should be specified in the format `projects/*/attestors/*`.
            Each attestor must exist before a policy can reference it. To add an
            attestor to a policy the principal issuing the policy change
            request must be able to read the attestor resource.
            Note: this field must be non-empty when the evaluation_mode field
            specifies REQUIRE_ATTESTATION, otherwise it must be empty.

        The **default_admission_rule** object supports the following:

          * `enforcementMode` (`pulumi.Input[str]`) - The action when a pod creation is denied by the admission rule.
          * `evaluationMode` (`pulumi.Input[str]`) - How this admission rule will be evaluated.
          * `requireAttestationsBies` (`pulumi.Input[list]`) - The resource names of the attestors that must attest to a
            container image. If the attestor is in a different project from the
            policy, it should be specified in the format `projects/*/attestors/*`.
            Each attestor must exist before a policy can reference it. To add an
            attestor to a policy the principal issuing the policy change
            request must be able to read the attestor resource.
            Note: this field must be non-empty when the evaluation_mode field
            specifies REQUIRE_ATTESTATION, otherwise it must be empty.
        """
        opts = pulumi.ResourceOptions.merge(opts, pulumi.ResourceOptions(id=id))

        __props__ = dict()

        __props__["admission_whitelist_patterns"] = admission_whitelist_patterns
        __props__["cluster_admission_rules"] = cluster_admission_rules
        __props__["default_admission_rule"] = default_admission_rule
        __props__["description"] = description
        __props__["global_policy_evaluation_mode"] = global_policy_evaluation_mode
        __props__["project"] = project
        return Policy(resource_name, opts=opts, __props__=__props__)
    def translate_output_property(self, prop):
        return tables._CAMEL_TO_SNAKE_CASE_TABLE.get(prop) or prop

    def translate_input_property(self, prop):
        return tables._SNAKE_TO_CAMEL_CASE_TABLE.get(prop) or prop




# coding=utf-8
# *** WARNING: this file was generated by the Pulumi Terraform Bridge (tfgen) Tool. ***
# *** Do not edit by hand unless you're certain you know what you are doing! ***

import json
import warnings
import pulumi
import pulumi.runtime
from typing import Union
from .. import utilities, tables

class Index(pulumi.CustomResource):
    collection: pulumi.Output[str]
    """
    The collection being indexed.
    """
    database: pulumi.Output[str]
    """
    The Firestore database id. Defaults to `"(default)"`.
    """
    fields: pulumi.Output[list]
    """
    The fields supported by this index. The last field entry is always for
    the field path `__name__`. If, on creation, `__name__` was not
    specified as the last field, it will be added automatically with the
    same direction as that of the last field defined. If the final field
    in a composite index is not directional, the `__name__` will be
    ordered `"ASCENDING"` (unless explicitly specified otherwise).  Structure is documented below.

      * `arrayConfig` (`str`) - Indicates that this field supports operations on arrayValues. Only one of `order` and `arrayConfig` can
        be specified.
      * `fieldPath` (`str`) - Name of the field.
      * `order` (`str`) - Indicates that this field supports ordering by the specified order or comparing using =, <, <=, >, >=.
        Only one of `order` and `arrayConfig` can be specified.
    """
    name: pulumi.Output[str]
    """
    A server defined name for this index. Format:
    'projects/{{project}}/databases/{{database}}/collectionGroups/{{collection}}/indexes/{{server_generated_id}}'
    """
    project: pulumi.Output[str]
    """
    The ID of the project in which the resource belongs.
    If it is not provided, the provider project is used.
    """
    query_scope: pulumi.Output[str]
    """
    The scope at which a query is run.
    """
    def __init__(__self__, resource_name, opts=None, collection=None, database=None, fields=None, project=None, query_scope=None, __props__=None, __name__=None, __opts__=None):
        """
        Cloud Firestore indexes enable simple and complex queries against documents in a database.
         This resource manages composite indexes and not single
        field indexes.


        To get more information about Index, see:

        * [API documentation](https://cloud.google.com/firestore/docs/reference/rest/v1/projects.databases.collectionGroups.indexes)
        * How-to Guides
            * [Official Documentation](https://cloud.google.com/firestore/docs/query-data/indexing)

        ## Example Usage

        ### Firestore Index Basic

        ```python
        import pulumi
        import pulumi_gcp as gcp

        my_index = gcp.firestore.Index("my-index",
            collection="chatrooms",
            fields=[
                {
                    "fieldPath": "name",
                    "order": "ASCENDING",
                },
                {
                    "fieldPath": "description",
                    "order": "DESCENDING",
                },
                {
                    "fieldPath": "__name__",
                    "order": "DESCENDING",
                },
            ],
            project="my-project-name")
        ```

        :param str resource_name: The name of the resource.
        :param pulumi.ResourceOptions opts: Options for the resource.
        :param pulumi.Input[str] collection: The collection being indexed.
        :param pulumi.Input[str] database: The Firestore database id. Defaults to `"(default)"`.
        :param pulumi.Input[list] fields: The fields supported by this index. The last field entry is always for
               the field path `__name__`. If, on creation, `__name__` was not
               specified as the last field, it will be added automatically with the
               same direction as that of the last field defined. If the final field
               in a composite index is not directional, the `__name__` will be
               ordered `"ASCENDING"` (unless explicitly specified otherwise).  Structure is documented below.
        :param pulumi.Input[str] project: The ID of the project in which the resource belongs.
               If it is not provided, the provider project is used.
        :param pulumi.Input[str] query_scope: The scope at which a query is run.

        The **fields** object supports the following:

          * `arrayConfig` (`pulumi.Input[str]`) - Indicates that this field supports operations on arrayValues. Only one of `order` and `arrayConfig` can
            be specified.
          * `fieldPath` (`pulumi.Input[str]`) - Name of the field.
          * `order` (`pulumi.Input[str]`) - Indicates that this field supports ordering by the specified order or comparing using =, <, <=, >, >=.
            Only one of `order` and `arrayConfig` can be specified.
        """
        if __name__ is not None:
            warnings.warn("explicit use of __name__ is deprecated", DeprecationWarning)
            resource_name = __name__
        if __opts__ is not None:
            warnings.warn("explicit use of __opts__ is deprecated, use 'opts' instead", DeprecationWarning)
            opts = __opts__
        if opts is None:
            opts = pulumi.ResourceOptions()
        if not isinstance(opts, pulumi.ResourceOptions):
            raise TypeError('Expected resource options to be a ResourceOptions instance')
        if opts.version is None:
            opts.version = utilities.get_version()
        if opts.id is None:
            if __props__ is not None:
                raise TypeError('__props__ is only valid when passed in combination with a valid opts.id to get an existing resource')
            __props__ = dict()

            if collection is None:
                raise TypeError("Missing required property 'collection'")
            __props__['collection'] = collection
            __props__['database'] = database
            if fields is None:
                raise TypeError("Missing required property 'fields'")
            __props__['fields'] = fields
            __props__['project'] = project
            __props__['query_scope'] = query_scope
            __props__['name'] = None
        super(Index, __self__).__init__(
            'gcp:firestore/index:Index',
            resource_name,
            __props__,
            opts)

    @staticmethod
    def get(resource_name, id, opts=None, collection=None, database=None, fields=None, name=None, project=None, query_scope=None):
        """
        Get an existing Index resource's state with the given name, id, and optional extra
        properties used to qualify the lookup.

        :param str resource_name: The unique name of the resulting resource.
        :param str id: The unique provider ID of the resource to lookup.
        :param pulumi.ResourceOptions opts: Options for the resource.
        :param pulumi.Input[str] collection: The collection being indexed.
        :param pulumi.Input[str] database: The Firestore database id. Defaults to `"(default)"`.
        :param pulumi.Input[list] fields: The fields supported by this index. The last field entry is always for
               the field path `__name__`. If, on creation, `__name__` was not
               specified as the last field, it will be added automatically with the
               same direction as that of the last field defined. If the final field
               in a composite index is not directional, the `__name__` will be
               ordered `"ASCENDING"` (unless explicitly specified otherwise).  Structure is documented below.
        :param pulumi.Input[str] name: A server defined name for this index. Format:
               'projects/{{project}}/databases/{{database}}/collectionGroups/{{collection}}/indexes/{{server_generated_id}}'
        :param pulumi.Input[str] project: The ID of the project in which the resource belongs.
               If it is not provided, the provider project is used.
        :param pulumi.Input[str] query_scope: The scope at which a query is run.

        The **fields** object supports the following:

          * `arrayConfig` (`pulumi.Input[str]`) - Indicates that this field supports operations on arrayValues. Only one of `order` and `arrayConfig` can
            be specified.
          * `fieldPath` (`pulumi.Input[str]`) - Name of the field.
          * `order` (`pulumi.Input[str]`) - Indicates that this field supports ordering by the specified order or comparing using =, <, <=, >, >=.
            Only one of `order` and `arrayConfig` can be specified.
        """
        opts = pulumi.ResourceOptions.merge(opts, pulumi.ResourceOptions(id=id))

        __props__ = dict()

        __props__["collection"] = collection
        __props__["database"] = database
        __props__["fields"] = fields
        __props__["name"] = name
        __props__["project"] = project
        __props__["query_scope"] = query_scope
        return Index(resource_name, opts=opts, __props__=__props__)
    def translate_output_property(self, prop):
        return tables._CAMEL_TO_SNAKE_CASE_TABLE.get(prop) or prop

    def translate_input_property(self, prop):
        return tables._SNAKE_TO_CAMEL_CASE_TABLE.get(prop) or prop




# coding=utf-8
# *** WARNING: this file was generated by the Pulumi Terraform Bridge (tfgen) Tool. ***
# *** Do not edit by hand unless you're certain you know what you are doing! ***

import json
import warnings
import pulumi
import pulumi.runtime
from typing import Union
from .. import utilities, tables

class Metric(pulumi.CustomResource):
    bucket_options: pulumi.Output[dict]
    """
    The bucketOptions are required when the logs-based metric is using a DISTRIBUTION value type and it
    describes the bucket boundaries used to create a histogram of the extracted values.  Structure is documented below.

      * `explicitBuckets` (`dict`) - Specifies a set of buckets with arbitrary widths.  Structure is documented below.
        * `bounds` (`list`) - The values must be monotonically increasing.

      * `exponentialBuckets` (`dict`) - Specifies an exponential sequence of buckets that have a width that is proportional to the value of
        the lower bound. Each bucket represents a constant relative uncertainty on a specific value in the bucket.  Structure is documented below.
        * `growthFactor` (`float`) - Must be greater than 1.
        * `numFiniteBuckets` (`float`) - Must be greater than 0.
        * `scale` (`float`) - Must be greater than 0.

      * `linearBuckets` (`dict`) - Specifies a linear sequence of buckets that all have the same width (except overflow and underflow).
        Each bucket represents a constant absolute uncertainty on the specific value in the bucket.  Structure is documented below.
        * `numFiniteBuckets` (`float`) - Must be greater than 0.
        * `offset` (`float`) - Lower bound of the first bucket.
        * `width` (`float`) - Must be greater than 0.
    """
    description: pulumi.Output[str]
    """
    A description of this metric, which is used in documentation. The maximum length of the
    description is 8000 characters.
    """
    filter: pulumi.Output[str]
    """
    An advanced logs filter (https://cloud.google.com/logging/docs/view/advanced-filters) which
    is used to match log entries.
    """
    label_extractors: pulumi.Output[dict]
    """
    A map from a label key string to an extractor expression which is used to extract data from a log
    entry field and assign as the label value. Each label key specified in the LabelDescriptor must
    have an associated extractor expression in this map. The syntax of the extractor expression is
    the same as for the valueExtractor field.
    """
    metric_descriptor: pulumi.Output[dict]
    """
    The metric descriptor associated with the logs-based metric.  Structure is documented below.

      * `display_name` (`str`) - A concise name for the metric, which can be displayed in user interfaces. Use sentence case
        without an ending period, for example "Request count". This field is optional but it is
        recommended to be set for any metrics associated with user-visible concepts, such as Quota.
      * `labels` (`list`) - The set of labels that can be used to describe a specific instance of this metric type. For
        example, the appengine.googleapis.com/http/server/response_latencies metric type has a label
        for the HTTP response code, response_code, so you can look at latencies for successful responses
        or just for responses that failed.  Structure is documented below.
        * `description` (`str`) - A description of this metric, which is used in documentation. The maximum length of the
          description is 8000 characters.
        * `key` (`str`) - The label key.
        * `valueType` (`str`) - The type of data that can be assigned to the label.

      * `metricKind` (`str`) - Whether the metric records instantaneous values, changes to a value, etc.
        Some combinations of metricKind and valueType might not be supported.
        For counter metrics, set this to DELTA.
      * `unit` (`str`) - The unit in which the metric value is reported. It is only applicable if the valueType is
        `INT64`, `DOUBLE`, or `DISTRIBUTION`. The supported units are a subset of
        [The Unified Code for Units of Measure](http://unitsofmeasure.org/ucum.html) standard
      * `valueType` (`str`) - The type of data that can be assigned to the label.
    """
    name: pulumi.Output[str]
    """
    The client-assigned metric identifier. Examples - "error_count", "nginx/requests".
    Metric identifiers are limited to 100 characters and can include only the following
    characters A-Z, a-z, 0-9, and the special characters _-.,+!*',()%/. The forward-slash
    character (/) denotes a hierarchy of name pieces, and it cannot be the first character
    of the name.
    """
    project: pulumi.Output[str]
    """
    The ID of the project in which the resource belongs.
    If it is not provided, the provider project is used.
    """
    value_extractor: pulumi.Output[str]
    """
    A valueExtractor is required when using a distribution logs-based metric to extract the values to
    record from a log entry. Two functions are supported for value extraction - EXTRACT(field) or
    REGEXP_EXTRACT(field, regex). The argument are 1. field - The name of the log entry field from which
    the value is to be extracted. 2. regex - A regular expression using the Google RE2 syntax
    (https://github.com/google/re2/wiki/Syntax) with a single capture group to extract data from the specified
    log entry field. The value of the field is converted to a string before applying the regex. It is an
    error to specify a regex that does not include exactly one capture group.
    """
    def __init__(__self__, resource_name, opts=None, bucket_options=None, description=None, filter=None, label_extractors=None, metric_descriptor=None, name=None, project=None, value_extractor=None, __props__=None, __name__=None, __opts__=None):
        """
        Logs-based metric can also be used to extract values from logs and create a a distribution
        of the values. The distribution records the statistics of the extracted values along with
        an optional histogram of the values as specified by the bucket options.


        To get more information about Metric, see:

        * [API documentation](https://cloud.google.com/logging/docs/reference/v2/rest/v2/projects.metrics/create)
        * How-to Guides
            * [Official Documentation](https://cloud.google.com/logging/docs/apis)

        ## Example Usage

        ### Logging Metric Basic

        ```python
        import pulumi
        import pulumi_gcp as gcp

        logging_metric = gcp.logging.Metric("loggingMetric",
            bucket_options={
                "linearBuckets": {
                    "numFiniteBuckets": 3,
                    "offset": 1,
                    "width": 1,
                },
            },
            filter="resource.type=gae_app AND severity>=ERROR",
            label_extractors={
                "mass": "EXTRACT(jsonPayload.request)",
                "sku": "EXTRACT(jsonPayload.id)",
            },
            metric_descriptor={
                "display_name": "My metric",
                "labels": [
                    {
                        "description": "amount of matter",
                        "key": "mass",
                        "valueType": "STRING",
                    },
                    {
                        "description": "Identifying number for item",
                        "key": "sku",
                        "valueType": "INT64",
                    },
                ],
                "metricKind": "DELTA",
                "unit": "1",
                "valueType": "DISTRIBUTION",
            },
            value_extractor="EXTRACT(jsonPayload.request)")
        ```

        ### Logging Metric Counter Basic

        ```python
        import pulumi
        import pulumi_gcp as gcp

        logging_metric = gcp.logging.Metric("loggingMetric",
            filter="resource.type=gae_app AND severity>=ERROR",
            metric_descriptor={
                "metricKind": "DELTA",
                "valueType": "INT64",
            })
        ```

        ### Logging Metric Counter Labels

        ```python
        import pulumi
        import pulumi_gcp as gcp

        logging_metric = gcp.logging.Metric("loggingMetric",
            filter="resource.type=gae_app AND severity>=ERROR",
            label_extractors={
                "mass": "EXTRACT(jsonPayload.request)",
            },
            metric_descriptor={
                "labels": [{
                    "description": "amount of matter",
                    "key": "mass",
                    "valueType": "STRING",
                }],
                "metricKind": "DELTA",
                "valueType": "INT64",
            })
        ```

        :param str resource_name: The name of the resource.
        :param pulumi.ResourceOptions opts: Options for the resource.
        :param pulumi.Input[dict] bucket_options: The bucketOptions are required when the logs-based metric is using a DISTRIBUTION value type and it
               describes the bucket boundaries used to create a histogram of the extracted values.  Structure is documented below.
        :param pulumi.Input[str] description: A description of this metric, which is used in documentation. The maximum length of the
               description is 8000 characters.
        :param pulumi.Input[str] filter: An advanced logs filter (https://cloud.google.com/logging/docs/view/advanced-filters) which
               is used to match log entries.
        :param pulumi.Input[dict] label_extractors: A map from a label key string to an extractor expression which is used to extract data from a log
               entry field and assign as the label value. Each label key specified in the LabelDescriptor must
               have an associated extractor expression in this map. The syntax of the extractor expression is
               the same as for the valueExtractor field.
        :param pulumi.Input[dict] metric_descriptor: The metric descriptor associated with the logs-based metric.  Structure is documented below.
        :param pulumi.Input[str] name: The client-assigned metric identifier. Examples - "error_count", "nginx/requests".
               Metric identifiers are limited to 100 characters and can include only the following
               characters A-Z, a-z, 0-9, and the special characters _-.,+!*',()%/. The forward-slash
               character (/) denotes a hierarchy of name pieces, and it cannot be the first character
               of the name.
        :param pulumi.Input[str] project: The ID of the project in which the resource belongs.
               If it is not provided, the provider project is used.
        :param pulumi.Input[str] value_extractor: A valueExtractor is required when using a distribution logs-based metric to extract the values to
               record from a log entry. Two functions are supported for value extraction - EXTRACT(field) or
               REGEXP_EXTRACT(field, regex). The argument are 1. field - The name of the log entry field from which
               the value is to be extracted. 2. regex - A regular expression using the Google RE2 syntax
               (https://github.com/google/re2/wiki/Syntax) with a single capture group to extract data from the specified
               log entry field. The value of the field is converted to a string before applying the regex. It is an
               error to specify a regex that does not include exactly one capture group.

        The **bucket_options** object supports the following:

          * `explicitBuckets` (`pulumi.Input[dict]`) - Specifies a set of buckets with arbitrary widths.  Structure is documented below.
            * `bounds` (`pulumi.Input[list]`) - The values must be monotonically increasing.

          * `exponentialBuckets` (`pulumi.Input[dict]`) - Specifies an exponential sequence of buckets that have a width that is proportional to the value of
            the lower bound. Each bucket represents a constant relative uncertainty on a specific value in the bucket.  Structure is documented below.
            * `growthFactor` (`pulumi.Input[float]`) - Must be greater than 1.
            * `numFiniteBuckets` (`pulumi.Input[float]`) - Must be greater than 0.
            * `scale` (`pulumi.Input[float]`) - Must be greater than 0.

          * `linearBuckets` (`pulumi.Input[dict]`) - Specifies a linear sequence of buckets that all have the same width (except overflow and underflow).
            Each bucket represents a constant absolute uncertainty on the specific value in the bucket.  Structure is documented below.
            * `numFiniteBuckets` (`pulumi.Input[float]`) - Must be greater than 0.
            * `offset` (`pulumi.Input[float]`) - Lower bound of the first bucket.
            * `width` (`pulumi.Input[float]`) - Must be greater than 0.

        The **metric_descriptor** object supports the following:

          * `display_name` (`pulumi.Input[str]`) - A concise name for the metric, which can be displayed in user interfaces. Use sentence case
            without an ending period, for example "Request count". This field is optional but it is
            recommended to be set for any metrics associated with user-visible concepts, such as Quota.
          * `labels` (`pulumi.Input[list]`) - The set of labels that can be used to describe a specific instance of this metric type. For
            example, the appengine.googleapis.com/http/server/response_latencies metric type has a label
            for the HTTP response code, response_code, so you can look at latencies for successful responses
            or just for responses that failed.  Structure is documented below.
            * `description` (`pulumi.Input[str]`) - A description of this metric, which is used in documentation. The maximum length of the
              description is 8000 characters.
            * `key` (`pulumi.Input[str]`) - The label key.
            * `valueType` (`pulumi.Input[str]`) - The type of data that can be assigned to the label.

          * `metricKind` (`pulumi.Input[str]`) - Whether the metric records instantaneous values, changes to a value, etc.
            Some combinations of metricKind and valueType might not be supported.
            For counter metrics, set this to DELTA.
          * `unit` (`pulumi.Input[str]`) - The unit in which the metric value is reported. It is only applicable if the valueType is
            `INT64`, `DOUBLE`, or `DISTRIBUTION`. The supported units are a subset of
            [The Unified Code for Units of Measure](http://unitsofmeasure.org/ucum.html) standard
          * `valueType` (`pulumi.Input[str]`) - The type of data that can be assigned to the label.
        """
        if __name__ is not None:
            warnings.warn("explicit use of __name__ is deprecated", DeprecationWarning)
            resource_name = __name__
        if __opts__ is not None:
            warnings.warn("explicit use of __opts__ is deprecated, use 'opts' instead", DeprecationWarning)
            opts = __opts__
        if opts is None:
            opts = pulumi.ResourceOptions()
        if not isinstance(opts, pulumi.ResourceOptions):
            raise TypeError('Expected resource options to be a ResourceOptions instance')
        if opts.version is None:
            opts.version = utilities.get_version()
        if opts.id is None:
            if __props__ is not None:
                raise TypeError('__props__ is only valid when passed in combination with a valid opts.id to get an existing resource')
            __props__ = dict()

            __props__['bucket_options'] = bucket_options
            __props__['description'] = description
            if filter is None:
                raise TypeError("Missing required property 'filter'")
            __props__['filter'] = filter
            __props__['label_extractors'] = label_extractors
            if metric_descriptor is None:
                raise TypeError("Missing required property 'metric_descriptor'")
            __props__['metric_descriptor'] = metric_descriptor
            __props__['name'] = name
            __props__['project'] = project
            __props__['value_extractor'] = value_extractor
        super(Metric, __self__).__init__(
            'gcp:logging/metric:Metric',
            resource_name,
            __props__,
            opts)

    @staticmethod
    def get(resource_name, id, opts=None, bucket_options=None, description=None, filter=None, label_extractors=None, metric_descriptor=None, name=None, project=None, value_extractor=None):
        """
        Get an existing Metric resource's state with the given name, id, and optional extra
        properties used to qualify the lookup.

        :param str resource_name: The unique name of the resulting resource.
        :param str id: The unique provider ID of the resource to lookup.
        :param pulumi.ResourceOptions opts: Options for the resource.
        :param pulumi.Input[dict] bucket_options: The bucketOptions are required when the logs-based metric is using a DISTRIBUTION value type and it
               describes the bucket boundaries used to create a histogram of the extracted values.  Structure is documented below.
        :param pulumi.Input[str] description: A description of this metric, which is used in documentation. The maximum length of the
               description is 8000 characters.
        :param pulumi.Input[str] filter: An advanced logs filter (https://cloud.google.com/logging/docs/view/advanced-filters) which
               is used to match log entries.
        :param pulumi.Input[dict] label_extractors: A map from a label key string to an extractor expression which is used to extract data from a log
               entry field and assign as the label value. Each label key specified in the LabelDescriptor must
               have an associated extractor expression in this map. The syntax of the extractor expression is
               the same as for the valueExtractor field.
        :param pulumi.Input[dict] metric_descriptor: The metric descriptor associated with the logs-based metric.  Structure is documented below.
        :param pulumi.Input[str] name: The client-assigned metric identifier. Examples - "error_count", "nginx/requests".
               Metric identifiers are limited to 100 characters and can include only the following
               characters A-Z, a-z, 0-9, and the special characters _-.,+!*',()%/. The forward-slash
               character (/) denotes a hierarchy of name pieces, and it cannot be the first character
               of the name.
        :param pulumi.Input[str] project: The ID of the project in which the resource belongs.
               If it is not provided, the provider project is used.
        :param pulumi.Input[str] value_extractor: A valueExtractor is required when using a distribution logs-based metric to extract the values to
               record from a log entry. Two functions are supported for value extraction - EXTRACT(field) or
               REGEXP_EXTRACT(field, regex). The argument are 1. field - The name of the log entry field from which
               the value is to be extracted. 2. regex - A regular expression using the Google RE2 syntax
               (https://github.com/google/re2/wiki/Syntax) with a single capture group to extract data from the specified
               log entry field. The value of the field is converted to a string before applying the regex. It is an
               error to specify a regex that does not include exactly one capture group.

        The **bucket_options** object supports the following:

          * `explicitBuckets` (`pulumi.Input[dict]`) - Specifies a set of buckets with arbitrary widths.  Structure is documented below.
            * `bounds` (`pulumi.Input[list]`) - The values must be monotonically increasing.

          * `exponentialBuckets` (`pulumi.Input[dict]`) - Specifies an exponential sequence of buckets that have a width that is proportional to the value of
            the lower bound. Each bucket represents a constant relative uncertainty on a specific value in the bucket.  Structure is documented below.
            * `growthFactor` (`pulumi.Input[float]`) - Must be greater than 1.
            * `numFiniteBuckets` (`pulumi.Input[float]`) - Must be greater than 0.
            * `scale` (`pulumi.Input[float]`) - Must be greater than 0.

          * `linearBuckets` (`pulumi.Input[dict]`) - Specifies a linear sequence of buckets that all have the same width (except overflow and underflow).
            Each bucket represents a constant absolute uncertainty on the specific value in the bucket.  Structure is documented below.
            * `numFiniteBuckets` (`pulumi.Input[float]`) - Must be greater than 0.
            * `offset` (`pulumi.Input[float]`) - Lower bound of the first bucket.
            * `width` (`pulumi.Input[float]`) - Must be greater than 0.

        The **metric_descriptor** object supports the following:

          * `display_name` (`pulumi.Input[str]`) - A concise name for the metric, which can be displayed in user interfaces. Use sentence case
            without an ending period, for example "Request count". This field is optional but it is
            recommended to be set for any metrics associated with user-visible concepts, such as Quota.
          * `labels` (`pulumi.Input[list]`) - The set of labels that can be used to describe a specific instance of this metric type. For
            example, the appengine.googleapis.com/http/server/response_latencies metric type has a label
            for the HTTP response code, response_code, so you can look at latencies for successful responses
            or just for responses that failed.  Structure is documented below.
            * `description` (`pulumi.Input[str]`) - A description of this metric, which is used in documentation. The maximum length of the
              description is 8000 characters.
            * `key` (`pulumi.Input[str]`) - The label key.
            * `valueType` (`pulumi.Input[str]`) - The type of data that can be assigned to the label.

          * `metricKind` (`pulumi.Input[str]`) - Whether the metric records instantaneous values, changes to a value, etc.
            Some combinations of metricKind and valueType might not be supported.
            For counter metrics, set this to DELTA.
          * `unit` (`pulumi.Input[str]`) - The unit in which the metric value is reported. It is only applicable if the valueType is
            `INT64`, `DOUBLE`, or `DISTRIBUTION`. The supported units are a subset of
            [The Unified Code for Units of Measure](http://unitsofmeasure.org/ucum.html) standard
          * `valueType` (`pulumi.Input[str]`) - The type of data that can be assigned to the label.
        """
        opts = pulumi.ResourceOptions.merge(opts, pulumi.ResourceOptions(id=id))

        __props__ = dict()

        __props__["bucket_options"] = bucket_options
        __props__["description"] = description
        __props__["filter"] = filter
        __props__["label_extractors"] = label_extractors
        __props__["metric_descriptor"] = metric_descriptor
        __props__["name"] = name
        __props__["project"] = project
        __props__["value_extractor"] = value_extractor
        return Metric(resource_name, opts=opts, __props__=__props__)
    def translate_output_property(self, prop):
        return tables._CAMEL_TO_SNAKE_CASE_TABLE.get(prop) or prop

    def translate_input_property(self, prop):
        return tables._SNAKE_TO_CAMEL_CASE_TABLE.get(prop) or prop

